@@ -1,6 +1,6 @@
 /*! \file cvss.c
  *  \brief Interface to Common Vulnerability Scoring System Version 2
- *  
+ *
  *  See details at http://nvd.nist.gov/cvss.cfm
  *
  */
@@ -38,9 +38,9 @@
 #include <math.h>
 #include <string.h>
 
-#include "public/cvss.h"
+#include "public/cvss_score.h"
 #include "cvss_priv.h"
-#include "../common/elements.h"
+#include "common/elements.h"
 
 #define CVSS_SUPPORTED "2.0"
 #define NS_VULN_STR BAD_CAST "vuln"
@@ -63,7 +63,7 @@ static struct cvss_metrics **cvss_impact_metricsptr(struct cvss_impact* impact,
     }
 }
 
-struct cvss_impact *cvss_impact_new(void) { return oscap_calloc(1, sizeof(struct cvss_metrics)); }
+struct cvss_impact *cvss_impact_new(void) { return calloc(1, sizeof(struct cvss_metrics)); }
 
 struct cvss_keytab_entry {
     enum cvss_key key;     // cvss vector component
@@ -242,8 +242,8 @@ struct cvss_impact *cvss_impact_new_from_vector(const char *cvss_vector)
     }
 
 cleanup:
-    oscap_free(vector_dup);
-    oscap_free(components);
+    free(vector_dup);
+    free(components);
     return impact;
 
 syntax_error:
@@ -332,7 +332,7 @@ char *cvss_impact_to_vector(const struct cvss_impact* impact)
     assert(impact != NULL);
 
     // eight characters per component, 14 components
-    char *result = oscap_calloc(1, sizeof(char) * 8 * 14);
+    char *result = calloc(1, sizeof(char) * 8 * 14);
     char *out = result;
 
     out = cvss_metrics_to_vector(impact->base_metrics, out);
@@ -362,10 +362,10 @@ static inline const struct cvss_valtab_entry *cvss_impact_entry(const struct cvs
 void cvss_impact_free(struct cvss_impact* impact)
 {
     if (impact) {
-        oscap_free(impact->base_metrics);
-        oscap_free(impact->temporal_metrics);
-        oscap_free(impact->environmental_metrics);
-        oscap_free(impact);
+        cvss_metrics_free(cvss_impact_get_base_metrics(impact));
+        cvss_metrics_free(cvss_impact_get_temporal_metrics(impact));
+        cvss_metrics_free(cvss_impact_get_environmental_metrics(impact));
+        free(impact);
     }
 }
 
@@ -484,7 +484,7 @@ void cvss_impact_describe(const struct cvss_impact *impact, FILE *f)
     char *vec = cvss_impact_to_vector(impact);
     if (vec) {
         fprintf(f, "CVSS vector: %s\n\n", vec);
-        oscap_free(vec);
+        free(vec);
     }
 
     if (impact->base_metrics) {
@@ -520,7 +520,7 @@ struct cvss_metrics *cvss_metrics_new(enum cvss_category category)
 {
     assert(category != CVSS_NONE);
 
-    struct cvss_metrics *metrics = oscap_calloc(1, sizeof(struct cvss_metrics));
+    struct cvss_metrics *metrics = calloc(1, sizeof(struct cvss_metrics));
     metrics->category = category;
     metrics->score = NAN;
     return metrics;
@@ -592,10 +592,8 @@ bool cvss_metrics_export(const struct cvss_metrics *m, xmlTextWriterPtr writer)
     if (!isnan(m->score)) {
         char *score_str = oscap_sprintf("%.1f", m->score);
         xmlTextWriterWriteElementNS(writer, NULL, BAD_CAST "score", NULL, BAD_CAST score_str);
-        oscap_free(score_str);
+        free(score_str);
     }
-    if (m->source) xmlTextWriterWriteElementNS(writer, NULL, BAD_CAST "source", NULL, BAD_CAST m->source);
-    if (m->generated_on_datetime) xmlTextWriterWriteElementNS(writer, NULL, BAD_CAST "generated-on-datetime", NULL, BAD_CAST m->generated_on_datetime);
 
     for (size_t i = 0; i < cvss_metrics_component_num(m); ++i) {
         const struct cvss_valtab_entry *val = cvss_valtab(m->category | i, m->metrics.ANY[i], NULL, NULL);
@@ -604,6 +602,10 @@ bool cvss_metrics_export(const struct cvss_metrics *m, xmlTextWriterPtr writer)
             xmlTextWriterWriteElementNS(writer, NULL, BAD_CAST key->xml_str, NULL, BAD_CAST val->xml_str);
     }
 
+    if (m->source) xmlTextWriterWriteElementNS(writer, NULL, BAD_CAST "source", NULL, BAD_CAST m->source);
+    if (m->generated_on_datetime) xmlTextWriterWriteElementNS(writer, NULL, BAD_CAST "generated-on-datetime", NULL, BAD_CAST m->generated_on_datetime);
+
+
     xmlTextWriterEndElement(writer);
 
     return true;
@@ -612,10 +614,10 @@ bool cvss_metrics_export(const struct cvss_metrics *m, xmlTextWriterPtr writer)
 void cvss_metrics_free(struct cvss_metrics* metrics)
 {
     if (metrics) {
-        oscap_free(metrics->source);
-        oscap_free(metrics->upgraded_from_version);
-        oscap_free(metrics->generated_on_datetime);
-        oscap_free(metrics);
+        free(metrics->source);
+        free(metrics->upgraded_from_version);
+        free(metrics->generated_on_datetime);
+        free(metrics);
     }
 }
 
