@@ -7,13 +7,13 @@
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software 
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Authors:
@@ -35,6 +35,7 @@
 #include "ds_sds_session_priv.h"
 #include "sds_priv.h"
 
+#include "common/debug_priv.h"
 #include "common/alloc.h"
 #include "common/_error.h"
 #include "common/util.h"
@@ -155,9 +156,13 @@ static xmlNodePtr _lookup_component_in_collection(xmlDocPtr doc, const char *com
 	return component;
 }
 
-static int ds_sds_dump_component_sce(xmlNode *script_node, const char *component_id, const char *filename)
+static int ds_sds_dump_component_sce(const xmlNode *script_node, const char *component_id, const char *filename)
 {
 	if (script_node) {
+		if (oscap_acquire_ensure_parent_dir(filename) < 0) {
+			oscap_seterr(OSCAP_EFAMILY_XML, "Error while creating script parent directory for file '%s' of (id='%s')", filename, component_id);
+			return -1;
+		}
 		// TODO: should we check whether the component is extended?
 		int fd;
 		xmlChar* text_contents = xmlNodeGetContent(script_node);
@@ -180,6 +185,7 @@ static int ds_sds_dump_component_sce(xmlNode *script_node, const char *component
 			oscap_seterr(OSCAP_EFAMILY_XML, "Failed to set executable permission on script (id='%s') that was split to '%s'.", component_id, filename);
 		}
 
+		dD("Successfully dumped script component (id='%s') to file '%s'.", component_id, filename);
 		fclose(output_file);
 		xmlFree(text_contents);
 		return 0;
@@ -191,47 +197,229 @@ static int ds_sds_dump_component_sce(xmlNode *script_node, const char *component
 	}
 }
 
-static int ds_sds_dump_component(const char* component_id, struct ds_sds_session *session, const char* filename, const char *relative_filepath)
+/**
+ * Load oscap source from file
+ * Filename is relatively to datastream file
+ */
+static struct oscap_source *load_referenced_source(const struct ds_sds_session *session, const char *filename)
 {
-	xmlDoc *doc = ds_sds_session_get_xmlDoc(session);
-	xmlNodePtr component = _lookup_component_in_collection(doc, component_id);
-	if (component == NULL)
-	{
-		oscap_seterr(OSCAP_EFAMILY_XML, "Component of given id '%s' was not found in the document.", component_id);
+	const char* readable_origin = ds_sds_session_get_readable_origin(session);
+	assert(readable_origin != NULL);
+	char* readable_origin_cp = oscap_strdup(readable_origin);
+
+	char* dir_name = dirname(readable_origin_cp);
+	char* full_path = oscap_sprintf("%s/%s", dir_name, filename);
+
+	struct oscap_source *source_file = oscap_source_new_from_file(full_path);
+
+	free(full_path);
+	free(readable_origin_cp);
+
+	return source_file;
+}
+
+static int ds_sds_register_sce(struct ds_sds_session *session, xmlNodePtr component_inner_root, const char* component_id, const char* target_filename_dirname, const char* relative_filepath)
+{
+	// the cast is safe to do because we are using the GNU basename, it doesn't
+	// modify the string
+	const char* file_basename = basename((char*)relative_filepath);
+	char *sce_filename = oscap_sprintf("%s/%s/%s",ds_sds_session_get_target_dir(session), target_filename_dirname, file_basename);
+	const int ret = ds_sds_dump_component_sce(component_inner_root->children, component_id, sce_filename);
+	free(sce_filename);
+	return ret;
+}
+
+static int ds_sds_register_xmlDoc(struct ds_sds_session *session, xmlDoc* doc, xmlNodePtr component_inner_root, const char *relative_filepath)
+{
+	xmlDoc *new_doc = ds_doc_from_foreign_node(component_inner_root, doc);
+	if (new_doc == NULL) {
 		return -1;
 	}
 
-	xmlNodePtr inner_root = node_get_child_element(component, NULL);
+	struct oscap_source *component_source = oscap_source_new_from_xmlDoc(new_doc, relative_filepath);
 
-	if (inner_root == NULL)
+	ds_sds_session_register_component_source(session, relative_filepath, component_source);
+	return 0; // TODO: Return value of ds_sds_session_register_component_source(). (commit message)
+}
+
+static int ds_sds_register_component(struct ds_sds_session *session, xmlDoc* doc, xmlNodePtr component_inner_root, const char* component_id, const char* target_filename_dirname, const char* relative_filepath)
+{
+	if (component_inner_root == NULL)
 	{
 		oscap_seterr(OSCAP_EFAMILY_XML, "Found component (id='%s') but it has no element contents, nothing to dump, skipping...", component_id);
 		return -1;
 	}
 
 	// If the inner root is script, we have to treat it in a special way
-	if (strcmp((const char*)inner_root->name, "script") == 0) {
-		int ret = ds_sds_dump_component_sce(inner_root->children, component_id, filename);
-		if (ret != 0) {
-			return ret;
-		}
+	if (strcmp((const char*)component_inner_root->name, "script") == 0) {
+		return ds_sds_register_sce(session, component_inner_root, component_id, target_filename_dirname, relative_filepath);
+	} else {
+		// Otherwise we create a new XML doc we will dump the contents to.
+		// We can't just dump node "innerXML" because namespaces have to be
+		// handled.
+		return ds_sds_register_xmlDoc(session, doc, component_inner_root, relative_filepath);
 	}
-	// Otherwise we create a new XML doc we will dump the contents to.
-	// We can't just dump node "innerXML" because namespaces have to be
-	// handled.
-	else {
-		xmlDoc *new_doc = ds_doc_from_foreign_node(inner_root, doc);
-		if (new_doc == NULL) {
-			return -1;
+}
+
+static xmlNodePtr ds_sds_get_component_root_by_id(xmlDoc *doc, const char* component_id)
+{
+	xmlNodePtr component;
+	if (component_id == NULL) {
+		component = (xmlNodePtr)doc;
+	} else {
+		component = _lookup_component_in_collection(doc, component_id);
+		if (component == NULL)
+		{
+			oscap_seterr(OSCAP_EFAMILY_XML, "Component of given id '%s' was not found in the document.", component_id);
+			return NULL;
 		}
-		struct oscap_source *source = oscap_source_new_from_xmlDoc(new_doc, relative_filepath);
-		ds_sds_session_register_component_source(session, relative_filepath, source);
 	}
 
+	return node_get_child_element(component, NULL);
+}
+
+static int ds_sds_dump_local_component(const char* component_id, struct ds_sds_session *session, const char *target_filename_dirname, const char *relative_filepath)
+{
+	xmlDoc *doc = ds_sds_session_get_xmlDoc(session);
+
+	xmlNodePtr inner_root = ds_sds_get_component_root_by_id(doc, component_id);
+
+	return ds_sds_register_component(session, doc, inner_root, component_id, target_filename_dirname, relative_filepath);
+}
+
+static int ds_sds_dump_file_component(const char* external_file, const char* component_id, struct ds_sds_session *session, const char *target_filename_dirname, const char *relative_filepath)
+{
+	int ret = 0;
+
+	struct oscap_source *source_file = load_referenced_source(session, external_file);
+	xmlDoc *doc = oscap_source_get_xmlDoc(source_file);
+
+	if (doc == NULL) {
+		ret = -1;
+		goto cleanup;
+	}
+
+	xmlNodePtr inner_root = ds_sds_get_component_root_by_id(doc, component_id);
+
+	if (ds_sds_register_component(session, doc, inner_root, component_id, target_filename_dirname, relative_filepath) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+
+	cleanup:
+		oscap_source_free(source_file);
+		return ret;
+}
+
+static int ds_dsd_dump_remote_component(const char* url, const char* component_id, struct ds_sds_session *session, const char *target_filename_dirname, const char *relative_filepath)
+{
+	int ret = 0;
+	size_t memory_size = 0;
+
+	ds_sds_session_remote_resources_progress(session)(false, "Downloading: %s ... ", url);
+
+	char* mem = oscap_acquire_url_download(url, &memory_size);
+	if (mem == NULL) {
+		ds_sds_session_remote_resources_progress(session)(false, "error\n", url);
+		return -1;
+	}
+
+	ds_sds_session_remote_resources_progress(session)(false, "ok\n", url);
+
+	struct oscap_source *source_file = oscap_source_new_take_memory(mem, memory_size, url);
+	xmlDoc *doc = oscap_source_get_xmlDoc(source_file);
+
+	if (doc == NULL) {
+		ret = -1;
+		goto cleanup;
+	}
+
+	xmlNodePtr inner_root = ds_sds_get_component_root_by_id(doc, component_id);
+
+	if (ds_sds_register_component(session, doc, inner_root, component_id, target_filename_dirname, relative_filepath) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+
+	cleanup:
+		oscap_source_free(source_file);
+		return ret;
+}
+
+static char *compose_target_filename_dirname(const char *relative_filepath, const char* sub_dir)
+{
+	char* filename_cpy = oscap_sprintf("./%s", relative_filepath);
+	char* file_reldir = dirname(filename_cpy);
+
+	char* target_filename_dirname = oscap_sprintf("%s/%s",sub_dir, file_reldir);
+	free(filename_cpy);
+
+	return target_filename_dirname;
+}
+
+static int ds_sds_dump_component_by_href(struct ds_sds_session *session, char* xlink_href, char *target_filename_dirname, const char* relative_filepath, char* cref_id, char **component_id)
+{
+	if (!xlink_href || strlen(xlink_href) < 2)
+	{
+		oscap_seterr(OSCAP_EFAMILY_XML, "No or invalid xlink:href attribute on given component-ref.");
+		return -1;
+	}
+
+	if (xlink_href[0] == '#')
+	{
+		*component_id = xlink_href + 1;
+
+		return ds_sds_dump_local_component(*component_id, session, target_filename_dirname, relative_filepath);
+
+	} else if (oscap_str_startswith(xlink_href, "file:")){
+
+		char* sep = strchr(xlink_href, '#');
+
+		const char *filename = xlink_href + strlen("file:");
+
+		if (sep == NULL) {
+			*component_id = NULL;
+		} else {
+			*sep = '\0';
+			*component_id = sep + 1;
+		}
+
+		return ds_sds_dump_file_component(filename, *component_id, session, target_filename_dirname, relative_filepath);
+
+	} else if (oscap_acquire_url_is_supported(xlink_href)){
+		char *sep = strchr(xlink_href, '#');
+
+		char* url = xlink_href;
+
+		if (sep == NULL) {
+			*component_id = NULL;
+		} else {
+			*sep = '\0';
+			*component_id = sep + 1;
+		}
+
+		if (!ds_sds_session_fetch_remote_resources(session)) {
+			static bool fetch_remote_resources_suggested = false;
+
+			if (!fetch_remote_resources_suggested) {
+				fetch_remote_resources_suggested = true;
+				ds_sds_session_remote_resources_progress(session)(true, "WARNING: Datastream component '%s' points out to the remote '%s'. "
+									"Use '--fetch-remote-resources' option to download it.\n", cref_id, url);
+			}
+
+			ds_sds_session_remote_resources_progress(session)(true, "WARNING: Skipping '%s' file which is referenced from datastream\n", url);
+			return -2;
+		}
+
+		return ds_dsd_dump_remote_component(url, *component_id, session, target_filename_dirname, relative_filepath);
+	} else {
+		oscap_seterr(OSCAP_EFAMILY_XML, "Unsupported type of xlink:href attribute on given component-ref - '%s'.", xlink_href);
+		return -1;
+	}
 	return 0;
 }
 
-int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session *session, const char* target_dir, const char* relative_filepath)
+int ds_sds_dump_component_ref_as(const xmlNodePtr component_ref, struct ds_sds_session *session, const char* sub_dir, const char* relative_filepath)
 {
 	char* cref_id = (char*)xmlGetProp(component_ref, BAD_CAST "id");
 	if (!cref_id)
@@ -242,29 +430,22 @@ int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session
 	}
 
 	char* xlink_href = (char*)xmlGetNsProp(component_ref, BAD_CAST "href", BAD_CAST xlink_ns_uri);
-	if (!xlink_href || strlen(xlink_href) < 2)
-	{
-		oscap_seterr(OSCAP_EFAMILY_XML, "No or invalid xlink:href attribute on given component-ref.");
-		xmlFree(cref_id);
-		xmlFree(xlink_href);
+
+	char* target_filename_dirname = compose_target_filename_dirname(relative_filepath, sub_dir);
+
+	char* component_id = NULL;
+
+	int ret = ds_sds_dump_component_by_href(session, xlink_href, target_filename_dirname, relative_filepath, cref_id, &component_id);
+
+	xmlFree(xlink_href);
+	xmlFree(cref_id);
+
+	if (ret != 0) {
+
+		free(target_filename_dirname);
 		return -1;
 	}
 
-	assert(xlink_href[0] == '#');
-	const char* component_id = xlink_href + 1 * sizeof(char);
-	char* filename_cpy = oscap_sprintf("./%s", relative_filepath);
-	char* file_reldir = dirname(filename_cpy);
-
-	// the cast is safe to do because we are using the GNU basename, it doesn't
-	// modify the string
-	const char* file_basename = basename((char*)relative_filepath);
-
-	const char* target_filename_dirname = oscap_sprintf("%s/%s", target_dir, file_reldir);
-	const char* target_filename = oscap_sprintf("%s/%s/%s", target_dir, file_reldir, file_basename);
-	ds_sds_dump_component(component_id, session, target_filename, relative_filepath);
-	oscap_free(target_filename);
-	oscap_free(filename_cpy);
-
 	xmlNodePtr catalog = node_get_child_element(component_ref, "catalog");
 	if (catalog)
 	{
@@ -283,9 +464,7 @@ int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session
 			if (!name)
 			{
 				oscap_seterr(OSCAP_EFAMILY_XML, "No 'name' attribute for a component referenced in the catalog of component '%s'.", component_id);
-				xmlFree(cref_id);
-				xmlFree(xlink_href);
-				oscap_free(target_filename_dirname);
+				free(target_filename_dirname);
 				return -1;
 			}
 
@@ -296,9 +475,7 @@ int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session
 				oscap_seterr(OSCAP_EFAMILY_XML, "No or invalid 'uri' attribute for a component referenced in the catalog of component '%s'.", component_id);
 				xmlFree(str_uri);
 				xmlFree(name);
-				xmlFree(cref_id);
-				xmlFree(xlink_href);
-				oscap_free(target_filename_dirname);
+				free(target_filename_dirname);
 				return -1;
 			}
 
@@ -311,9 +488,7 @@ int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session
 				oscap_seterr(OSCAP_EFAMILY_XML, "component-ref with given id '%s' wasn't found in the document! We are looking for it because it's in the catalog of component '%s'.", str_uri + 1 * sizeof(char), component_id);
 				xmlFree(str_uri);
 				xmlFree(name);
-				xmlFree(cref_id);
-				xmlFree(xlink_href);
-				oscap_free(target_filename_dirname);
+				free(target_filename_dirname);
 				return -1;
 			}
 			xmlFree(str_uri);
@@ -321,9 +496,7 @@ int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session
 			if (ds_sds_dump_component_ref_as(cat_component_ref, session, target_filename_dirname, name) != 0)
 			{
 				xmlFree(name);
-				xmlFree(cref_id);
-				xmlFree(xlink_href);
-				oscap_free(target_filename_dirname);
+				free(target_filename_dirname);
 				return -1; // no need to call oscap_seterr here, it's already set
 			}
 
@@ -331,14 +504,13 @@ int ds_sds_dump_component_ref_as(xmlNodePtr component_ref, struct ds_sds_session
 		}
 	}
 
-	oscap_free(target_filename_dirname);
-	xmlFree(cref_id);
-	xmlFree(xlink_href);
+	free(target_filename_dirname);
+
 
 	return 0;
 }
 
-int ds_sds_dump_component_ref(xmlNodePtr component_ref, struct ds_sds_session *session)
+int ds_sds_dump_component_ref(const xmlNodePtr component_ref, struct ds_sds_session *session)
 {
 	char* cref_id = (char*)xmlGetProp(component_ref, BAD_CAST "id");
 	if (!cref_id)
@@ -348,7 +520,7 @@ int ds_sds_dump_component_ref(xmlNodePtr component_ref, struct ds_sds_session *s
 		return -1;
 	}
 
-	int result = ds_sds_dump_component_ref_as(component_ref, session, ds_sds_session_get_target_dir(session), cref_id);
+	int result = ds_sds_dump_component_ref_as(component_ref, session, ".", cref_id);
 	xmlFree(cref_id);
 
 	// if result is -1, oscap_seterr was already called, no need to call it again
@@ -432,18 +604,18 @@ static inline int ds_sds_compose_component_add_script_content(xmlNode *component
 	long int length = ftell(f);
 	fseek(f, 0, SEEK_SET);
 	if (length >= 0) {
-		char* buffer = oscap_alloc((length + 1) * sizeof(char));
+		char* buffer = malloc((length + 1) * sizeof(char));
 		if (fread(buffer, length, 1, f) != 1) {
 			oscap_seterr(OSCAP_EFAMILY_GLIBC, "Error while reading from file '%s'.", filepath);
 			fclose(f);
-			oscap_free(buffer);
+			free(buffer);
 			return -1;
 		}
 		fclose(f);
 		buffer[length] = '\0';
 		xmlNsPtr local_ns = xmlNewNs(component, BAD_CAST sce_xccdf_ns_uri, BAD_CAST "oscap-sce-xccdf-stream");
 		xmlNewTextChild(component, local_ns, BAD_CAST "script", BAD_CAST buffer);
-		oscap_free(buffer);
+		free(buffer);
 		return 0;
 	} else {
 		oscap_seterr(OSCAP_EFAMILY_GLIBC, "No data read from file '%s'.", filepath);
@@ -572,13 +744,13 @@ static int ds_sds_compose_catalog_has_uri(xmlDocPtr doc, xmlNodePtr catalog, con
 	// this is done for performance reasons
 	xpathCtx->node = catalog;
 
-	const char* expression = oscap_sprintf("cat:uri[@uri = '%s']", uri);
+	char *expression = oscap_sprintf("cat:uri[@uri = '%s']", uri);
 
 	xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression(
 			BAD_CAST expression,
 			xpathCtx);
 
-	oscap_free(expression);
+	free(expression);
 
 	if (xpathObj == NULL)
 	{
@@ -609,7 +781,7 @@ static char* ds_sds_mangle_filepath(const char* filepath)
 
 	// the string will grow 2x the size in the worst case (every char is /)
 	// TODO: We can do better than this by counting the slashes
-	char* ret = oscap_alloc(strlen(filepath) * sizeof(char) * 2);
+	char* ret = malloc(strlen(filepath) * sizeof(char) * 2);
 
 	const char* src_it = filepath;
 	char* dst_it = ret;
@@ -621,6 +793,10 @@ static char* ds_sds_mangle_filepath(const char* filepath)
 			*dst_it++ = '-';
 			*dst_it++ = '-';
 		}
+		else if (*src_it == '@') {
+			*dst_it++ = '-';
+			*dst_it++ = '-';
+		}
 		else
 		{
 			*dst_it++ = *src_it;
@@ -660,9 +836,6 @@ static int ds_sds_compose_add_component_dependencies(xmlDocPtr doc, xmlNodePtr d
 		return -1;
 	}
 
-	//xmlXPathRegisterNs(xpathCtx, BAD_CAST "cdf11", BAD_CAST "http://checklists.nist.gov/xccdf/1.1");
-	//xmlXPathRegisterNs(xpathCtx, BAD_CAST "cdf12", BAD_CAST "http://checklists.nist.gov/xccdf/1.2");
-
 	xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression(
 			// we want robustness and support for future versions, this expression
 			// retrieves check-content-refs from any namespace
@@ -710,8 +883,11 @@ static int ds_sds_compose_add_component_dependencies(xmlDocPtr doc, xmlNodePtr d
 					continue;
 				}
 
-				char* real_path = (strcmp(dir, "") == 0 || strcmp(dir, ".") == 0) ?
-					oscap_strdup(href) : oscap_sprintf("%s/%s", dir, href);
+				// skip over file:// if it's used in the file href
+				const char *altered_href = oscap_str_startswith(href, "file://") ? href + 7 : href;
+
+				char* real_path = (strcmp(dir, "") == 0 || strcmp(dir, ".") == 0 || altered_href[0] == '/') ?
+					oscap_strdup(altered_href) : oscap_sprintf("%s/%s", dir, altered_href);
 
 				char* mangled_path = ds_sds_mangle_filepath(real_path);
 				char* cref_id = oscap_sprintf("scap_org.open-scap_cref_%s", mangled_path);
@@ -719,19 +895,19 @@ static int ds_sds_compose_add_component_dependencies(xmlDocPtr doc, xmlNodePtr d
 				int counter = 0;
 				while (ds_sds_find_component_ref(datastream, cref_id) != NULL) {
 					// While the given component ID already exists in the document.
-					oscap_free(cref_id);
+					free(cref_id);
 					cref_id = oscap_sprintf("scap_org.open-scap_cref_%s%03d", mangled_path, counter++);
 				}
-				oscap_free(mangled_path);
+				free(mangled_path);
 
 				char* uri = oscap_sprintf("#%s", cref_id);
 
 				// we don't want duplicated uri elements in the catalog
 				if (ds_sds_compose_catalog_has_uri(doc, catalog, uri) == 0)
 				{
-					oscap_free(uri);
-					oscap_free(cref_id);
-					oscap_free(real_path);
+					free(uri);
+					free(cref_id);
+					free(real_path);
 					xmlFree(href);
 					continue;
 				}
@@ -744,9 +920,9 @@ static int ds_sds_compose_add_component_dependencies(xmlDocPtr doc, xmlNodePtr d
 					xmlAddChild(catalog, catalog_uri);
 				}
 
-				oscap_free(cref_id);
-				oscap_free(uri);
-				oscap_free(real_path);
+				free(cref_id);
+				free(uri);
+				free(real_path);
 				xmlFree(href);
 
 				if (ret < 0) {
@@ -759,7 +935,7 @@ static int ds_sds_compose_add_component_dependencies(xmlDocPtr doc, xmlNodePtr d
 		}
 
 		oscap_htable_free0(exported);
-		oscap_free(filepath_cpy);
+		free(filepath_cpy);
 	}
 
 	xmlXPathFreeObject(xpathObj);
@@ -784,13 +960,13 @@ static int ds_sds_compose_has_component_ref(xmlDocPtr doc, xmlNodePtr datastream
 	// this is done for performance reasons
 	xpathCtx->node = datastream;
 
-	const char* expression = oscap_sprintf("*/ds:component-ref[@xlink:href = '#%s' and @id = '%s']", filepath, cref_id);
+	char *expression = oscap_sprintf("*/ds:component-ref[@xlink:href = '#%s' and @id = '%s']", filepath, cref_id);
 
 	xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression(
 			BAD_CAST expression,
 			xpathCtx);
 
-	oscap_free(expression);
+	free(expression);
 
 	if (xpathObj == NULL)
 	{
@@ -871,7 +1047,7 @@ int ds_sds_compose_add_component_with_ref(xmlDocPtr doc, xmlNodePtr datastream,
 			return -1;
 		}
 	}
-	else if (doc_type == OSCAP_DOCUMENT_OVAL_DEFINITIONS)
+	else if (doc_type == OSCAP_DOCUMENT_OVAL_DEFINITIONS || doc_type == OSCAP_DOCUMENT_OCIL)
 	{
 		cref_parent = node_get_child_element(datastream, "checks");
 	}
@@ -892,12 +1068,12 @@ int ds_sds_compose_add_component_with_ref(xmlDocPtr doc, xmlNodePtr datastream,
 	int counter = 0;
 	while (_lookup_component_in_collection(doc, comp_id) != NULL) {
 		// While a component of the given ID already exists, generate a new one
-		oscap_free(comp_id);
+		free(comp_id);
 		comp_id = oscap_sprintf("scap_org.open-scap_%scomp_%s%03d",
 			extended_component ? "e" : "", mangled_filepath, counter++);
 	}
 
-	oscap_free(mangled_filepath);
+	free(mangled_filepath);
 
 	result = ds_sds_compose_add_component_internal(doc, datastream, filepath, comp_id, extended_component);
 	if (result == 0) {
@@ -905,9 +1081,9 @@ int ds_sds_compose_add_component_with_ref(xmlDocPtr doc, xmlNodePtr datastream,
 		xmlAddChild(cref, cref_catalog);
 		xmlSetProp(cref, BAD_CAST "id", BAD_CAST cref_id);
 
-		const char* xlink_href = oscap_sprintf("#%s", comp_id);
+		char *xlink_href = oscap_sprintf("#%s", comp_id);
 		xmlSetNsProp(cref, xlink_ns, BAD_CAST "href", BAD_CAST xlink_href);
-		oscap_free(xlink_href);
+		free(xlink_href);
 
 		if (xmlAddChild(cref_parent, cref) == NULL) {
 			oscap_seterr(OSCAP_EFAMILY_XML, "Failed to add component-ref/@id='%s' to the DataStream.", cref_id);
@@ -915,7 +1091,7 @@ int ds_sds_compose_add_component_with_ref(xmlDocPtr doc, xmlNodePtr datastream,
 		}
 	}
 
-	oscap_free(comp_id);
+	free(comp_id);
 	// the source data stream XSD requires either no catalog or a non-empty one
 	if (cref_catalog->children == NULL)
 	{
@@ -936,12 +1112,12 @@ int ds_sds_compose_add_component(const char *target_datastream, const char *data
 	}
 	xmlNodePtr datastream = ds_sds_lookup_datastream_in_collection(doc, datastream_id);
 	if (datastream == NULL) {
-		const char* error = datastream_id ?
+		char *error = datastream_id ?
 			oscap_sprintf("Could not find any datastream of id '%s'", datastream_id) :
 			oscap_sprintf("Could not find any datastream inside the file");
 
 		oscap_seterr(OSCAP_EFAMILY_XML, error);
-		oscap_free(error);
+		free(error);
 		oscap_source_free(sds_source);
 		return 1;
 	}
@@ -949,13 +1125,13 @@ int ds_sds_compose_add_component(const char *target_datastream, const char *data
 	char* mangled_path = ds_sds_mangle_filepath(new_component);
 
 	char* cref_id = oscap_sprintf("scap_org.open-scap_cref_%s", mangled_path);
-	oscap_free(mangled_path);
+	free(mangled_path);
 	if (ds_sds_compose_add_component_with_ref(doc, datastream, new_component, cref_id) != 0) {
-		oscap_free(cref_id);
+		free(cref_id);
 		oscap_source_free(sds_source);
 		return 1;
 	}
-	oscap_free(cref_id);
+	free(cref_id);
 
 	if (oscap_source_save_as(sds_source, NULL) != 0) {
 		oscap_seterr(OSCAP_EFAMILY_GLIBC, "Error saving source datastream to '%s'.", target_datastream);
@@ -982,9 +1158,9 @@ int ds_sds_compose_from_xccdf(const char* xccdf_file, const char* target_datastr
 	char* mangled_xccdf_file = ds_sds_mangle_filepath(xccdf_file);
 	char* collection_id = oscap_sprintf("scap_org.open-scap_collection_from_xccdf_%s", mangled_xccdf_file);
 	xmlSetProp(root, BAD_CAST "id", BAD_CAST collection_id);
-	oscap_free(collection_id);
+	free(collection_id);
 
-	xmlSetProp(root, BAD_CAST "schematron-version", BAD_CAST "1.0");
+	xmlSetProp(root, BAD_CAST "schematron-version", BAD_CAST "1.2");
 
 	// we will need this namespace later when creating component-ref
 	// dependency catalog
@@ -995,7 +1171,7 @@ int ds_sds_compose_from_xccdf(const char* xccdf_file, const char* target_datastr
 
 	char* datastream_id = oscap_sprintf("scap_org.open-scap_datastream_from_xccdf_%s", mangled_xccdf_file);
 	xmlSetProp(datastream, BAD_CAST "id", BAD_CAST datastream_id);
-	oscap_free(datastream_id);
+	free(datastream_id);
 
 	xmlSetProp(datastream, BAD_CAST "scap-version", BAD_CAST "1.2");
 
@@ -1017,11 +1193,11 @@ int ds_sds_compose_from_xccdf(const char* xccdf_file, const char* target_datastr
 	if (ds_sds_compose_add_component_with_ref(doc, datastream, xccdf_file, cref_id) != 0)
 	{
 		// oscap_seterr already called
-		oscap_free(cref_id);
-		oscap_free(mangled_xccdf_file);
+		free(cref_id);
+		free(mangled_xccdf_file);
 		return -1;
 	}
-	oscap_free(cref_id);
+	free(cref_id);
 
 	// the XSD of source data stream enforces that the collection elements are
 	// not empty if they are there, we will therefore now removes collections
@@ -1048,7 +1224,7 @@ int ds_sds_compose_from_xccdf(const char* xccdf_file, const char* target_datastr
 		xmlFreeNode(extended_components);
 	}
 
-	oscap_free(mangled_xccdf_file);
+	free(mangled_xccdf_file);
 
 	if (xmlSaveFileEnc(target_datastream, doc, "utf-8") == -1)
 	{
