@@ -109,7 +109,7 @@ oval_agent_session_t * oval_agent_new_session(struct oval_definition_model *mode
 	if (ret != 0) {
 		oval_probe_session_destroy(ag_sess->psess);
 		oval_syschar_model_free(ag_sess->sys_model);
-		oscap_free(ag_sess);
+		free(ag_sess);
 		return NULL;
 	}
 	oval_syschar_model_set_sysinfo(ag_sess->sys_model, sysinfo);
@@ -228,8 +228,10 @@ int oval_agent_reset_session(oval_agent_session_t * ag_sess) {
         	oval_generator_set_product_name(generator, ag_sess->product_name);
 	}
 
-	oval_probe_session_destroy(ag_sess->psess);
-	ag_sess->psess = oval_probe_session_new(ag_sess->sys_model);
+	/* We have to reset probe_session inplace, because
+	 * ag_sess->res_model points to old probe_session
+	 * and we are not able to update the reference clearly */
+	oval_probe_session_reinit(ag_sess->psess, ag_sess->sys_model);
 
 	return 0;
 }
@@ -296,12 +298,12 @@ const char * oval_agent_get_filename(oval_agent_session_t * ag_sess) {
 
 void oval_agent_destroy_session(oval_agent_session_t * ag_sess) {
 	if (ag_sess != NULL) {
-		oscap_free(ag_sess->product_name);
+		free(ag_sess->product_name);
 		oval_probe_session_destroy(ag_sess->psess);
 		oval_syschar_model_free(ag_sess->sys_model);
 		oval_results_model_free(ag_sess->res_model);
-	        oscap_free(ag_sess->filename);
-		oscap_free(ag_sess);
+	        free(ag_sess->filename);
+		free(ag_sess);
 	}
 }
 
@@ -506,7 +508,7 @@ int oval_agent_resolve_variables(struct oval_agent_session * session, struct xcc
 			dI("Adding external variable %s.", name);
 		} else {
 			/* Skip this variable (we assume it has same values otherwise conflict was detected) */
-			dW("Skipping external variable %s.", name);
+			dI("Skipping external variable %s.", name);
 		}
         } else {
                 dW("Variable %s does not exist, skipping.", name);
