@@ -65,7 +65,7 @@ typedef struct oval_definition_model {
  * */
 struct oval_definition_model *oval_definition_model_new()
 {
-	oval_definition_model_t *newmodel = (oval_definition_model_t *) oscap_alloc(sizeof(oval_definition_model_t));
+	oval_definition_model_t *newmodel = (oval_definition_model_t *) malloc(sizeof(oval_definition_model_t));
 	if (newmodel == NULL)
 		return NULL;
 
@@ -136,10 +136,10 @@ void oval_definition_model_free(struct oval_definition_model *model)
 					   (oscap_destruct_func) oval_variable_model_free);
 
 	        if (model->schema != NULL)
-			oscap_free(model->schema);
+			free(model->schema);
 
 		oval_generator_free(model->generator);
-		oscap_free(model);
+		free(model);
 	}
 }
 
@@ -497,12 +497,14 @@ xmlNode *oval_definition_model_to_dom(struct oval_definition_model *definition_m
 	xmlNs *ns_unix = xmlNewNs(root_node, OVAL_DEFINITIONS_UNIX_NS, BAD_CAST "unix-def");
 	xmlNs *ns_ind = xmlNewNs(root_node, OVAL_DEFINITIONS_IND_NS, BAD_CAST "ind-def");
 	xmlNs *ns_lin = xmlNewNs(root_node, OVAL_DEFINITIONS_LIN_NS, BAD_CAST "lin-def");
+	xmlNs *ns_win = xmlNewNs(root_node, OVAL_DEFINITIONS_WIN_NS, BAD_CAST "win-def");
 	xmlNs *ns_defntns = xmlNewNs(root_node, OVAL_DEFINITIONS_NAMESPACE, NULL);
 
 	xmlSetNs(root_node, ns_common);
 	xmlSetNs(root_node, ns_unix);
 	xmlSetNs(root_node, ns_ind);
 	xmlSetNs(root_node, ns_lin);
+	xmlSetNs(root_node, ns_win);
 	xmlSetNs(root_node, ns_defntns);
 
 	/* Always report the generator */
