@@ -1399,7 +1399,7 @@ SEXP_t *SEXP_list_sort(SEXP_t *list, int(*compare)(const SEXP_t *, const SEXP_t
 
         list_it[0].block = SEXP_LCASTP(v_dsc.mem)->b_addr;
 
-        dI("Sorting blocks & building iterator array");
+        dD("Sorting blocks & building iterator array");
 
         while (list_it[list_it_count - 1].block != NULL) {
                 /* initialize the rest of the iterator */
@@ -1413,7 +1413,7 @@ SEXP_t *SEXP_list_sort(SEXP_t *list, int(*compare)(const SEXP_t *, const SEXP_t
 
                 /* reallocate the iterator array if needed */
                 if (list_it_count == list_it_alloc) {
-                        dI("Reallocating iterator array: %z -> %z",
+                        dD("Reallocating iterator array: %z -> %z",
                            list_it_alloc, list_it_alloc + SEXP_LISTIT_ARRAY_INC);
 
                         list_it_alloc += SEXP_LISTIT_ARRAY_INC;
@@ -1426,7 +1426,7 @@ SEXP_t *SEXP_list_sort(SEXP_t *list, int(*compare)(const SEXP_t *, const SEXP_t
         }
 
         --list_it_count;
-        dI("Iterator count = %zu", list_it_count);
+        dD("Iterator count = %zu", list_it_count);
 
         if (list_it_count > 0) {
                 /*
@@ -1463,7 +1463,7 @@ SEXP_t *SEXP_list_sort(SEXP_t *list, int(*compare)(const SEXP_t *, const SEXP_t
 								    list_it[min_i].count - 1, sizeof(SEXP_t), (void *)&tmp_v,
 								    (int(*)(void *, void *))compare, &dst_i);
 
-						dI("dst_i = %zu", dst_i);
+						dD("dst_i = %zu", dst_i);
 
 						/* make place for the old value in the min. value source block */
 						memmove(list_it[min_i].block->memb, list_it[min_i].block->memb + 1,
@@ -2036,6 +2036,8 @@ static int __SEXP_sizeof_lmemb (const SEXP_t *s_exp, size_t *sz)
                 }
 
                 ret = SEXP_rawval_lblk_cb ((uintptr_t)SEXP_LCASTP(v_dsc.mem)->b_addr, (int(*)(SEXP_t *, void *))__SEXP_sizeof_lmemb, sz, 1);
+                (*sz) += sizeof (SEXP_valhdr_t) + v_dsc.hdr->size;
+                break;
         }
         case SEXP_VALTYPE_NUMBER:
         case SEXP_VALTYPE_STRING:
