@@ -102,32 +102,32 @@ static int file_cb (const char *p, const char *f, void *ptr)
 	}
 
         SEXP_init(&xattr_name);
-retry_list:
-        /* estimate the size of the buffer */
-        xattr_count = llistxattr(st_path, NULL, 0);
 
-        if (xattr_count == 0)
-                return (0);
+	do {
+		/* estimate the size of the buffer */
+		xattr_count = llistxattr(st_path, NULL, 0);
+
+		if (xattr_count == 0)
+				return (0);
+
+		if (xattr_count < 0) {
+				dI("FAIL: llistxattr(%s, %p, %zu): errno=%u, %s.", errno, strerror(errno));
+				return 0;
+		}
+
+		/* allocate space for xattr names */
+		xattr_buflen = xattr_count;
+		xattr_buf    = realloc(xattr_buf, sizeof(char) * xattr_buflen);
+
+		/* fill the buffer */
+		xattr_count = llistxattr(st_path, xattr_buf, xattr_buflen);
+
+		/* check & retry if needed */
+	} while (errno == ERANGE);
 
         if (xattr_count < 0) {
                 dI("FAIL: llistxattr(%s, %p, %zu): errno=%u, %s.", errno, strerror(errno));
-                return 0;
-        }
-
-        /* allocate space for xattr names */
-        xattr_buflen = xattr_count;
-        xattr_buf    = oscap_realloc(xattr_buf, sizeof(char) * xattr_buflen);
-
-        /* fill the buffer */
-        xattr_count = llistxattr(st_path, xattr_buf, xattr_buflen);
-
-        /* check & retry if needed */
-        if (errno == ERANGE)
-                goto retry_list;
-
-        if (xattr_count < 0) {
-                dI("FAIL: llistxattr(%s, %p, %zu): errno=%u, %s.", errno, strerror(errno));
-                oscap_free(xattr_buf);
+                free(xattr_buf);
         }
 
         /* update lastpath if needed */
@@ -151,12 +151,24 @@ retry_list:
                         xattr_vallen = lgetxattr(st_path, xattr_buf + i, NULL, 0);
                 retry_value:
                         if (xattr_vallen >= 0) {
-                                xattr_val    = oscap_realloc(xattr_val, sizeof(char) * xattr_vallen);
+				// Check possible buffer overflow
+				if (sizeof(char) * (xattr_vallen + 1) <= sizeof(char) * xattr_vallen) {
+					dE("Attribute is too long.");
+					abort();
+				}
+
+				// Allocate buffer, '+1' is for trailing '\0'
+ 				xattr_val    = realloc(xattr_val, sizeof(char) * (xattr_vallen + 1));
+
+				// we don't want to override space for '\0' by call of 'lgetxattr'
+				// we pass only 'xattr_vallen' instead of 'xattr_vallen + 1'
                                 xattr_vallen = lgetxattr(st_path, xattr_buf + i, xattr_val, xattr_vallen);
 
                                 if (xattr_vallen < 0 || errno == ERANGE)
                                         goto retry_value;
 
+				xattr_val[xattr_vallen] = '\0';
+
                                 item = probe_item_create(OVAL_UNIX_FILEEXTENDEDATTRIBUTE, NULL,
                                                          "filepath", OVAL_DATATYPE_STRING, f == NULL ? NULL : st_path,
                                                          "path",     OVAL_DATATYPE_SEXP,  &gr_lastpath,
@@ -165,7 +177,7 @@ retry_list:
                                                          "value",          OVAL_DATATYPE_STRING, xattr_val,
                                                          NULL);
 
-                                oscap_free(xattr_val);
+                                free(xattr_val);
                         } else {
                                 dI("FAIL: lgetxattr(%s, %s, NULL, 0): errno=%u, %s.", errno, strerror(errno));
 
@@ -173,7 +185,7 @@ retry_list:
                                 probe_item_setstatus(item, SYSCHAR_STATUS_ERROR);
 
                                 if (xattr_val != NULL)
-                                        oscap_free(xattr_val);
+                                        free(xattr_val);
                         }
 
                         probe_item_collect(args->ctx, item); /* XXX: handle ENOMEM */
@@ -184,9 +196,10 @@ retry_list:
                 /* skip to next name */
                 while (i < xattr_buflen && xattr_buf[i] != '\0')
                         ++i;
-        } while (xattr_buf + i != xattr_buf + xattr_buflen - 1);
+		++i;
+        } while (xattr_buf + i < xattr_buf + xattr_buflen - 1);
 
-        oscap_free(xattr_buf);
+        free(xattr_buf);
 
         return (0);
 }
@@ -195,6 +208,8 @@ static pthread_mutex_t __file_probe_mutex;
 
 void *probe_init (void)
 {
+	probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, PROBE_OFFLINE_CHROOT);
+
 	SEXP_init(&gr_lastpath);
 
         /*
@@ -210,7 +225,6 @@ void *probe_init (void)
 	probe_setoption(PROBEOPT_VARREF_HANDLING, false, "path");
 	probe_setoption(PROBEOPT_VARREF_HANDLING, false, "filename");
 #endif
-		probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, PROBE_OFFLINE_CHROOT);
         return (NULL);
 }
 
@@ -285,7 +299,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
 	cbargs.error    = 0;
         cbargs.attr_ent = attribute_;
 
-	if ((ofts = oval_fts_open(path, filename, filepath, behaviors)) != NULL) {
+	if ((ofts = oval_fts_open(path, filename, filepath, behaviors, probe_ctx_getresult(ctx))) != NULL) {
 		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
 			file_cb(ofts_ent->path, ofts_ent->file, &cbargs);
 			oval_ftsent_free(ofts_ent);
