@@ -24,13 +24,18 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_RPM46
 int rpmErrorCb (rpmlogRec rec, rpmlogCallbackData data)
 {
 	dE("RPM: %s", rpmlogRecMessage(rec));
 	return RPMLOG_DEFAULT;
 }
+#endif
 
 void rpmLibsPreload()
 {
-	rpmReadConfigFiles(NULL, NULL);
+	// Don't load rpmrc files. The are useless for us,
+	// because we only need to preload libraries
+	const char* rcfiles = "";
+	rpmReadConfigFiles(rcfiles, NULL);
 }
