@@ -58,6 +58,8 @@
 #include "probe/entcmp.h"
 #include "alloc.h"
 #include "common/debug_priv.h"
+#include <probe/probe.h>
+#include <probe/option.h>
 
 /* Convenience structure for the results being reported */
 struct result_info {
@@ -137,6 +139,13 @@ static int read_password(SEXP_t *un_ent, probe_ctx *ctx, oval_schema_version_t o
         return 0;
 }
 
+void *probe_init(void)
+{
+	// Intentionally commented-out, see commit message
+	// probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, PROBE_OFFLINE_CHROOT);
+	return (NULL);
+}
+
 int probe_main(probe_ctx *ctx, void *arg)
 {
 	SEXP_t *ent, *obj;
