@@ -97,7 +97,7 @@ struct xiconf_attr {
 #define OPRES_EFAULT  3 /* Invalid memory access */
 #define OPRES_ELIMIT  4 /* Limit reached */
 
-typedef struct {
+typedef struct xiconf_service {
 	/*
 	 * Attributes needed by the xinetd probe
 	 */
@@ -124,6 +124,8 @@ typedef struct {
 	bool unlisted;
 	bool rpc;
 
+	struct xiconf_service *next;
+
 #if 0 /* Unused attributes; Maybe we will need them in some future version. */
 	bool groups;
 	bool mdns;
@@ -361,7 +363,7 @@ static xiconf_t *xiconf_new(void)
 	xiconf_t *xiconf;
 
 	xiconf = oscap_talloc(xiconf_t);
-	xiconf->cfile = oscap_alloc(sizeof(xiconf_file_t *));
+	xiconf->cfile = malloc(sizeof(xiconf_file_t *));
 	xiconf->count = 0;
 	xiconf->stree = rbt_str_new();
 	xiconf->ttree = rbt_str_new();
@@ -397,6 +399,8 @@ static xiconf_service_t *xiconf_service_new(void)
 	service->unlisted = 0;
 	service->rpc      = 0;
 
+	service->next = NULL;
+
 	return (service);
 }
 
@@ -405,7 +409,7 @@ static void xiconf_service_free(xiconf_service_t *service)
         if (service == NULL)
                 return;
 
-#define NONNULL_FREE(p) if (service->p != NULL) oscap_free(service->p)
+#define NONNULL_FREE(p) if (service->p != NULL) free(service->p)
 
         if (service->id != service->name)
                 NONNULL_FREE(id);
@@ -420,7 +424,12 @@ static void xiconf_service_free(xiconf_service_t *service)
         NONNULL_FREE(server_args);
         NONNULL_FREE(only_from);
         NONNULL_FREE(no_access);
-	oscap_free(service);
+
+	if (service->next != NULL) {
+		xiconf_service_free(service->next);
+	}
+
+	free(service);
 }
 
 static void xiconf_stree_free_cb(struct rbt_str_node *n)
@@ -431,7 +440,7 @@ static void xiconf_stree_free_cb(struct rbt_str_node *n)
 static void xiconf_ttree_free_cb(struct rbt_str_node *n)
 {
         xiconf_strans_free(n->data);
-	oscap_free(n->key);
+	free(n->key);
 }
 
 void xiconf_free(xiconf_t *xiconf)
@@ -444,24 +453,24 @@ void xiconf_free(xiconf_t *xiconf)
 	for (i = 0; i < xiconf->count; ++i) {
 		if (xiconf->cfile[i]) {
 			if (xiconf->cfile[i]->cpath)
-				oscap_free(xiconf->cfile[i]->cpath);
+				free(xiconf->cfile[i]->cpath);
 			if (xiconf->cfile[i]->inmem)
-				oscap_free(xiconf->cfile[i]->inmem);
-			oscap_free(xiconf->cfile[i]);
+				free(xiconf->cfile[i]->inmem);
+			free(xiconf->cfile[i]);
 		}
 	}
 
-	oscap_free(xiconf->cfile);
+	free(xiconf->cfile);
 
         rbt_str_free_cb(xiconf->stree, xiconf_stree_free_cb);
         rbt_str_free_cb(xiconf->ttree, xiconf_ttree_free_cb);
 
         if (xiconf->defaults != NULL) {
-	        oscap_free(xiconf->defaults->name);
-	        oscap_free(xiconf->defaults);
+	        free(xiconf->defaults->name);
+	        free(xiconf->defaults);
         }
 
-	oscap_free(xiconf);
+	free(xiconf);
 
 	return;
 }
@@ -483,7 +492,7 @@ static xiconf_file_t *xiconf_read(const char *path, int flags)
 	}
 
 	if ((st.st_mode & S_IFMT) != S_IFREG) {
-		dE("Not a regular file: %s\n", path);
+		dE("Not a regular file: %s", path);
 		close (fd);
 		return (NULL);
 	}
@@ -514,12 +523,12 @@ static xiconf_file_t *xiconf_read(const char *path, int flags)
 	{
 		/* fallback method - copy the contents into memory */
 
-		file->inmem = oscap_alloc(file->inlen);
+		file->inmem = malloc(file->inlen);
 
 		if (read (file->fd, file->inmem, file->inlen) != (ssize_t)file->inlen) {
 			/* Can't read the contents of the file */
 			close (fd);
-			oscap_free(file);
+			free(file);
 			return (NULL);
 		}
 
@@ -545,30 +554,30 @@ static int xiconf_add_cfile(xiconf_t *xiconf, const char *path, int depth)
 	xiconf_file_t *xifile;
 
 	if (xiconf->count + 1 > XICFG_PARSER_MAXFILECOUNT) {
-		dE("include count limit reached: %u\n", XICFG_PARSER_MAXFILECOUNT);
+		dE("include count limit reached: %u", XICFG_PARSER_MAXFILECOUNT);
 		return (-1);
 	}
 
-	dI("Reading included file: %s\n", path);
+	dI("Reading included file: %s", path);
 	xifile = xiconf_read (path, 0);
 
 	if (xifile == NULL) {
-		dW("Failed to read file: %s\n", path);
+		dW("Failed to read file: %s", path);
 		return (-1);
 	}
 
 	xifile->depth = depth;
-	xiconf->cfile = oscap_realloc(xiconf->cfile, sizeof(xiconf_file_t *) * ++xiconf->count);
+	xiconf->cfile = realloc(xiconf->cfile, sizeof(xiconf_file_t *) * ++xiconf->count);
 	xiconf->cfile[xiconf->count - 1] = xifile;
 
-	dI("Added new file to the cfile queue: %s; fi=%zu\n", path, xiconf->count - 1);
+	dI("Added new file to the cfile queue: %s; fi=%zu", path, xiconf->count - 1);
 
 	return (0);
 }
 
 #define tmpbuf_def(size) char __tmpbuf[size]
-#define tmpbuf_get(size) (((sizeof __tmpbuf)/sizeof(char))<(size)?oscap_alloc(sizeof(char)*(size)):__tmpbuf)
-#define tmpbuf_free(ptr) do { if ((ptr) != __tmpbuf) oscap_free(ptr); (ptr) = NULL; } while(0)
+#define tmpbuf_get(size) (((sizeof __tmpbuf)/sizeof(char))<(size)?malloc(sizeof(char)*(size)):__tmpbuf)
+#define tmpbuf_free(ptr) do { if ((ptr) != __tmpbuf) free(ptr); (ptr) = NULL; } while(0)
 
 xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 {
@@ -715,18 +724,18 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 				buffer[l_size] = '\0';
 
 				if (*inclarg == '\0') {
-					dW("Found includedir directive without an argument!\n");
+					dW("Found includedir directive without an argument!");
 					break;
 				}
 
 				if (xifile->depth + 1 > max_depth) {
-					dE("include depth limit reached: %u\n", max_depth);
+					dE("include depth limit reached: %u", max_depth);
 					break;
 				}
 
 				switch (inctype) {
 				case XICONF_INCTYPE_FILE:
-					dI("includefile: %s\n", pathbuf);
+					dI("includefile: %s", pathbuf);
 
 					if (xiconf_add_cfile (xiconf, pathbuf, xifile->depth + 1) != 0)
 						continue;
@@ -737,11 +746,11 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 					DIR           *dirfp;
 					struct dirent  dent, *dentp = NULL;
 
-					dI("includedir open: %s\n", inclarg);
+					dI("includedir open: %s", inclarg);
 					dirfp = opendir (inclarg);
 
 					if (dirfp == NULL) {
-						dW("Can't open includedir: %s; %d, %s.\n", inclarg, errno, strerror (errno));
+						dW("Can't open includedir: %s; %d, %s.", inclarg, errno, strerror (errno));
 						break;
 					}
 
@@ -753,7 +762,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 							pathbuf[incllen++] =  '/';
 							pathbuf[incllen  ] = '\0';
 						} else {
-							dE("Length of the includedir argument is out of range: len=%zu, max=%zu\n",
+							dE("Length of the includedir argument is out of range: len=%zu, max=%zu",
 							   incllen, PATH_MAX);
 							closedir(dirfp);
 							break;
@@ -762,7 +771,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 
 					for (;;) {
 						if (readdir_r (dirfp, &dent, &dentp) != 0) {
-							dW("Can't read directory: %s; %d, %s.\n", inclarg, errno, strerror (errno));
+							dW("Can't read directory: %s; %d, %s.", inclarg, errno, strerror (errno));
 							break;
 						}
 
@@ -772,7 +781,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 						if (fnmatch ("*~",  dent.d_name, FNM_PATHNAME) == 0 ||
 						    fnmatch ("*.*", dent.d_name, FNM_PATHNAME) == 0)
 						{
-							dI("Skipping: %s\n", dent.d_name);
+							dI("Skipping: %s", dent.d_name);
 							continue;
 						}
 
@@ -782,14 +791,14 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 							continue;
 					}
 
-					dI("includedir close: %s\n", inclarg);
+					dI("includedir close: %s", inclarg);
 					closedir(dirfp);
 					break;
 				}}
 				break;
 			}
 			default:
-				dE("Don't know how to parse the line: %s\n", buffer);
+				dE("Don't know how to parse the line: %s", buffer);
 				tmpbuf_free(buffer);
 
 				return (NULL);
@@ -804,52 +813,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 
 int xiconf_update(xiconf_t *xiconf)
 {
-	dI("Not implemented yet.\n");
-	return (0);
-}
-
-static int xiconf_service_merge_and_free(xiconf_service_t *dst, xiconf_service_t *src)
-{
-	register size_t i;
-
-	for (i = 0; i < XIATTR_TABLE_COUNT; ++i) {
-		if (xiattr_table[i].pass_arg == XIATTR_OPARG_LOCAL &&
-		    xiattr_table[i].sections  & XIATTR_SECTION_SRV &&
-		    xiattr_table[i].op_merge != NULL)
-		{
-			if (xiattr_table[i].op_merge (xiattr_ptr(dst, xiattr_table[i].offset),
-						      xiattr_ptr(src, xiattr_table[i].offset), 0) != 0)
-			{
-				dE("Merge failed. Merge operation returned a non-zero value.\n");
-				    return (-1);
-			}
-		}
-	}
-
-	/*
-	 * Merge service names. If there are already assigned names in dst and src, they must be equal.
-	 */
-	if (src->name != NULL) {
-		if (dst->name != NULL) {
-			if (strcmp(src->name, dst->name) != 0) {
-				dE("Merge not possible: different service names: %s (dst) != %s (src)\n",
-				   dst->name, src->name);
-				return (-1);
-			}
-
-			if (dst->id != src->name)
-				oscap_free(src->name);
-		} else
-			dst->name = src->name;
-	}
-
-	dst->def_disabled |= src->def_disabled;
-	dst->def_enabled  |= src->def_enabled;
-	dst->internal     |= src->internal;
-	dst->unlisted     |= src->unlisted;
-	dst->rpc          |= src->rpc;
-
-	oscap_free(src);
+	dI("Not implemented yet.");
 	return (0);
 }
 
@@ -865,7 +829,7 @@ static int xiconf_service_merge_defaults(xiconf_service_t *dst, xiconf_service_t
 			if (xiattr_table[i].op_merge (xiattr_ptr(dst, xiattr_table[i].offset),
 						      xiattr_ptr(def, xiattr_table[i].offset), XIATTR_MERGE_DEFAULTS) != 0)
 			{
-				dE("Merge failed. Merge operation returned a non-zero value.\n");
+				dE("Merge failed. Merge operation returned a non-zero value.");
 				    return (-1);
 			}
 		}
@@ -889,10 +853,10 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 	tmpbuf_def(128);
 
 	if (type == XICONF_SECTION_DEFAULTS && xiconf->defaults != NULL) {
-		dE("defaults section was already parsed!\n");
+		dE("defaults section was already parsed!");
 		return (-1);
 	} else if (type == XICONF_SECTION_SERVICE && name == NULL) {
-		dE("Don't know how to handle anonymous service sections!\n");
+		dE("Don't know how to handle anonymous service sections!");
 		return (-1);
 	}
 
@@ -989,7 +953,7 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 			xiattr = oscap_bfind(xiattr_table, XIATTR_TABLE_COUNT, sizeof(struct xiconf_attr), key, &xiattr_cmp);
 
 			if (xiattr == NULL) {
-				dW("Unknown keyword: %s\n", key);
+				dW("Unknown keyword: %s", key);
 #if XICFG_PARSER_IGNORE_UNKNOWN != 1
 				goto fail;
 #else
@@ -998,7 +962,7 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 			}
 
 			if (!(type & xiattr->sections)) {
-				dW("Don't know how to handle attribute %s in this sections type (%d)\n",
+				dW("Don't know how to handle attribute %s in this sections type (%d)",
 				   key, type);
 #if XICFG_PARSER_IGNORE_INVSECT != 1
 				goto fail;
@@ -1010,11 +974,11 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 			switch (xiattr->pass_arg) {
 			case XIATTR_OPARG_LOCAL:
 				opvar = (void *)xiattr_ptr(snew, xiattr->offset);
-				dI("local opvar\n");
+				dI("local opvar");
 				break;
 			case XIATTR_OPARG_GLOBAL:
 				opvar = (void *)xiconf;
-				dI("global opvar\n");
+				dI("global opvar");
 				break;
 			default:
 				abort ();
@@ -1024,21 +988,21 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 				opfun = xiattr->op_assign;
 				opval = op + 1;
 
-				dI("assign(%p): var=%s (at %p+%zu = %p), val=%s\n",
+				dI("assign(%p): var=%s (at %p+%zu = %p), val=%s",
 				   opfun, xiattr->name, snew, xiattr->offset, opvar, opval);
 
 			} else if (*op == '+' && *(op+1) == '=') {
 				opfun = xiattr->op_insert;
 				opval = op + 2;
 
-				dI("insert(%p): var=%s (at %p+%zu = %p), val=%s\n",
+				dI("insert(%p): var=%s (at %p+%zu = %p), val=%s",
 				   opfun, xiattr->name, snew, xiattr->offset, opvar, opval);
 
 			} else if (*op == '-' && *(op+1) == '=') {
 				opfun = xiattr->op_remove;
 				opval = op + 2;
 
-				dI("remove(%p): var=%s (at %p+%zu = %p), val=%s\n",
+				dI("remove(%p): var=%s (at %p+%zu = %p), val=%s",
 				   opfun, xiattr->name, snew, xiattr->offset, opvar, opval);
 			} else
 				goto fail;
@@ -1057,12 +1021,12 @@ int xiconf_parse_section(xiconf_t *xiconf, xiconf_file_t *xifile, int type, char
 
 finish_section:
 	if (buffer != NULL) {
-		dW("Line buffer not freed; freeing now...\n");
+		dW("Line buffer not freed; freeing now...");
 		tmpbuf_free(buffer);
 	}
 
 	if (key != NULL) {
-		dW("key not freed; freeing now...\n");
+		dW("key not freed; freeing now...");
 		free(key);
 	}
 
@@ -1086,80 +1050,75 @@ finish_section:
 
 		if (scur == NULL) {
 			if (rbt_str_add (xiconf->stree, snew->id, snew) != 0) {
-				dE("Can't add service (%s) into the service tree (%p)\n",
+				dE("Can't add service (%s) into the service tree (%p)",
 				   snew->id, xiconf->stree);
 
 				xiconf_service_free(snew);
 				return (-1);
 			}
-
-			scur = snew;
-
-			if (scur->protocol == NULL) {
-				// First try to guess the protocol from the socket_type setting
-				if (scur->socket_type != NULL) {
-					// dgram => udp
-					if (strcmp(scur->socket_type, "dgram") == 0) {
-						scur->protocol = strdup("udp");
-					}
-					// stream => tcp
-					else if (strcmp(scur->socket_type, "stream") == 0) {
-						scur->protocol = strdup("tcp");
-					}
+		} else {
+			if (scur->next == NULL) {
+				scur->next = snew;
+			} else {
+				snew->next = scur->next;
+				scur->next = snew;
+			}
+		}
+		scur = snew;
+		if (scur->protocol == NULL) {
+			// First try to guess the protocol from the socket_type setting
+			if (scur->socket_type != NULL) {
+				// dgram => udp
+				if (strcmp(scur->socket_type, "dgram") == 0) {
+					scur->protocol = strdup("udp");
 				}
-				// If we still don't know the protocol, then get the default from /etc/services
-				if (scur->protocol == NULL) {
-					struct servent *service = getservbyname(scur->name, NULL);
-					dI("protocol is empty, trying to guess from /etc/services for %s\n", scur->name);
-					if (service != NULL) {
-						scur->protocol = strdup(service->s_proto);
-						dI("service %s has default protocol=%s\n", scur->name, scur->protocol);
-					}
-					endservent();
+				// stream => tcp
+				else if (strcmp(scur->socket_type, "stream") == 0) {
+					scur->protocol = strdup("tcp");
 				}
 			}
-			if (scur->port == 0) {
-				struct servent *service = getservbyname(scur->name, scur->protocol);
-				dI("port not set, trying to guess from /etc/services for %s\n", scur->name);
+			// If we still don't know the protocol, then get the default from /etc/services
+			if (scur->protocol == NULL) {
+				struct servent *service = getservbyname(scur->name, NULL);
+				dI("protocol is empty, trying to guess from /etc/services for %s", scur->name);
 				if (service != NULL) {
-					if (service->s_port > 0 && service->s_port < 65536) {
-						scur->port = ntohs((uint16_t)service->s_port);
-						dI("service %s has default port=%hu/%s\n",
-						   scur->name, scur->port, scur->protocol);
-					}
+					scur->protocol = strdup(service->s_proto);
+					dI("service %s has default protocol=%s", scur->name, scur->protocol);
 				}
 				endservent();
 			}
-			if (scur->type != 0) {
-				// Check that the type is one of the allowed values by OVAL.
-				// The xinetd configuration acually allows any combination of
-				// the values. In such case, we set the type to an empty string.
-				const char *type_enum[] = { "RPC", "INTERNAL", "UNLISTED", "TCPMUX", "TCPMUXPLUS", NULL };
-				bool type_enum_match = false;
-				size_t i;
-				for (i = 0; type_enum[i]; ++i) {
-					if (strcmp(scur->type, type_enum[i]) == 0) {
-						type_enum_match = true;
-						break;
-					}
-				}
-				if (!type_enum_match) {
-					dE("The value of the type setting does not match"
-					   " any of the allowed values by OVAL: %s\n", scur->type);
-					scur->type = "";
+		}
+		if (scur->port == 0) {
+			struct servent *service = getservbyname(scur->name, scur->protocol);
+			dI("port not set, trying to guess from /etc/services for %s", scur->name);
+			if (service != NULL) {
+				if (service->s_port > 0 && service->s_port < 65536) {
+					scur->port = ntohs((uint16_t)service->s_port);
+					dI("service %s has default port=%hu/%s",
+					   scur->name, scur->port, scur->protocol);
 				}
 			}
-		} else {
-			dI("Merge(%p, %p)\n", scur, snew);
-
-			if (xiconf_service_merge_and_free(scur, snew) != 0) {
-				dE("Failed to merge service tree records %p and %p (id=%s)\n",
-				   scur, snew, snew->id);
-				xiconf_service_free(snew);
-				return (-1);
+			endservent();
+		}
+		if (scur->type != 0) {
+			// Check that the type is one of the allowed values by OVAL.
+			// The xinetd configuration acually allows any combination of
+			// the values. In such case, we set the type to an empty string.
+			const char *type_enum[] = { "RPC", "INTERNAL", "UNLISTED", "TCPMUX", "TCPMUXPLUS", NULL };
+			bool type_enum_match = false;
+			size_t i;
+			for (i = 0; type_enum[i]; ++i) {
+				if (strcmp(scur->type, type_enum[i]) == 0) {
+					type_enum_match = true;
+					break;
+				}
+			}
+			if (!type_enum_match) {
+				dE("The value of the type setting does not match"
+				   " any of the allowed values by OVAL: %s", scur->type);
+				scur->type = "";
 			}
 		}
-
 		/*
 		 * Add entry to the ttree for (name, protocol) -> (id) translation
 		 * (in case it's not already there)
@@ -1171,23 +1130,23 @@ finish_section:
 		rbt_str_get(xiconf->ttree, st_key, (void *)&st);
 
 		if (st == NULL) {
-			dI("new strans record: k=%s\n", st_key);
+			dI("new strans record: k=%s", st_key);
 
 			st = oscap_talloc (xiconf_strans_t);
 			st->cnt = 1;
-			st->srv = oscap_alloc (sizeof (xiconf_service_t *));
+			st->srv = malloc (sizeof (xiconf_service_t *));
 			st->srv[0] = scur;
 
 			if (rbt_str_add (xiconf->ttree, strdup(st_key), st) != 0) {
-				dE("Can't add strans record (k=%s) into the strans tree (%p)\n",
+				dE("Can't add strans record (k=%s) into the strans tree (%p)",
 				   st_key, xiconf->ttree);
 				return (-1);
 			}
 		} else {
-			dI("adding new strans record to an exiting one: k=%s, cnt=%u+1\n",
+			dI("adding new strans record to an exiting one: k=%s, cnt=%u+1",
 			   st_key, st->cnt);
 
-			st->srv = oscap_realloc(st->srv, sizeof (xiconf_service_t *) * ++(st->cnt));
+			st->srv = realloc(st->srv, sizeof (xiconf_service_t *) * ++(st->cnt));
 			st->srv[st->cnt - 1] = scur;
 		}
 
@@ -1196,7 +1155,7 @@ finish_section:
 		 * that happen to have a default value defined.
 		 */
 		if (xiconf_service_merge_defaults(scur, xiconf->defaults) != 0) {
-			dE("Failed to merge service (%p, id=%s) with defaults (%p)\n",
+			dE("Failed to merge service (%p, id=%s) with defaults (%p)",
 			   scur, scur->id, xiconf->defaults);
 			return (-1);
 		}
@@ -1213,12 +1172,12 @@ finish_section:
 fail:
 	tmpbuf_free(buffer);
 	if (key) {
-		dW("fail: key not freed; freeing now...\n");
+		dW("fail: key not freed; freeing now...");
 		free(key);
 	}
 
 	if (snew->name) {
-		dW("fail: snew->name not freed; freeing now...\n");
+		dW("fail: snew->name not freed; freeing now...");
 		free(snew->name);
 	}
 	free(snew);
@@ -1267,7 +1226,7 @@ xiconf_strans_t *xiconf_dump(xiconf_t *xiconf)
 
 	res = oscap_talloc(xiconf_strans_t);
 	res->cnt = rbt_str_size(xiconf->stree);
-	res->srv = oscap_alloc(sizeof(xiconf_service_t *) * res->cnt);
+	res->srv = malloc(sizeof(xiconf_service_t *) * res->cnt);
 
 	rbt_str_walk_inorder2(xiconf->stree, xiconf_dump_cb, (void *)res, 0);
 
@@ -1281,8 +1240,8 @@ void xiconf_strans_free(xiconf_strans_t *strans)
         if (strans == NULL)
                 return;
 
-        oscap_free(strans->srv);
-        oscap_free(strans);
+        free(strans->srv);
+        free(strans);
         return;
 }
 
@@ -1354,7 +1313,7 @@ int op_merge_str(void *dst, void *src, int type)
 	char **b = (char **)src;
 
 	if (*a != NULL) {
-		dE("Cannot merge str value: dst=%p (%s), src=%p (%s)\n", dst, *a, src, *b);
+		dE("Cannot merge str value: dst=%p (%s), src=%p (%s)", dst, *a, src, *b);
 		return (-1);
 	} else
 		*a = *b;
@@ -1372,10 +1331,10 @@ int op_assign_strl(void *var, char *val)
 	if (string_array != NULL) {
 		// Destroy previous array state
 		while(string_array[string_array_size]) {
-			oscap_free(string_array[string_array_size]);
+			free(string_array[string_array_size]);
 			++string_array_size;
 		}
-		oscap_free(string_array);
+		free(string_array);
 		string_array = NULL;
 		string_array_size = 0;
 	}
@@ -1387,8 +1346,8 @@ int op_assign_strl(void *var, char *val)
 		if (*tok == '\0') {
 			continue;
 		}
-		dI("Adding new member to string array: %s\n", tok);
-		string_array = oscap_realloc(string_array, sizeof(char *) * (++string_array_size + 1));
+		dI("Adding new member to string array: %s", tok);
+		string_array = realloc(string_array, sizeof(char *) * (++string_array_size + 1));
 		string_array[string_array_size-1] = strdup(tok);
 		string_array[string_array_size] = NULL;
 	}
@@ -1408,7 +1367,7 @@ int op_insert_strl(void *var, char *val)
 		while(string_array[string_array_size]) {
 			++string_array_size;
 		}
-		dI("String array has %zu members\n", string_array_size);
+		dI("String array has %zu members", string_array_size);
 	}
 	str = val;
 	while ((tok = strsep(&str, " ")) != NULL) {
@@ -1418,8 +1377,8 @@ int op_insert_strl(void *var, char *val)
 		if (*tok == '\0') {
 			continue;
 		}
-		dI("Adding new member to string array: %s\n", tok);
-		string_array = oscap_realloc(string_array, sizeof(char *) * (++string_array_size + 1));
+		dI("Adding new member to string array: %s", tok);
+		string_array = realloc(string_array, sizeof(char *) * (++string_array_size + 1));
 		string_array[string_array_size-1] = strdup(tok);
 		string_array[string_array_size] = NULL;
 	}
@@ -1442,7 +1401,7 @@ int op_remove_strl(void *var, char *val)
 		while(string_array[string_array_size]) {
 			++string_array_size;
 		}
-		dI("String array has %zu members\n", string_array_size);
+		dI("String array has %zu members", string_array_size);
 	}
 
 	if (string_array_size < 1) {
@@ -1450,7 +1409,7 @@ int op_remove_strl(void *var, char *val)
 		return 0;
 	} else {
 		// Allocate the new string array
-		newstr_array = oscap_alloc(sizeof(char *) * (string_array_size + 1));
+		newstr_array = malloc(sizeof(char *) * (string_array_size + 1));
 	}
 
 	// Create an array of strings to be removed from the array
@@ -1464,8 +1423,8 @@ int op_remove_strl(void *var, char *val)
 		if (*tok == '\0') {
 			continue;
 		}
-		dI("Adding new member to string array: %s\n", tok);
-		valstr_array = oscap_realloc(valstr_array, sizeof(char *) * (++valstr_array_size + 1));
+		dI("Adding new member to string array: %s", tok);
+		valstr_array = realloc(valstr_array, sizeof(char *) * (++valstr_array_size + 1));
 		valstr_array[valstr_array_size-1] = tok;
 		valstr_array[valstr_array_size] = NULL;
 	}
@@ -1477,12 +1436,12 @@ int op_remove_strl(void *var, char *val)
 
 		for (valstr_array_pos = 0; valstr_array[valstr_array_pos]; ++valstr_array_pos) {
 			char *delete_val = valstr_array[valstr_array_pos];
-			dI("Removing: %s\n", delete_val);
+			dI("Removing: %s", delete_val);
 			// Destroy the string if it matches a string from the value string array
 			// Otherwise move it to the new string array
-			dI("cmp: %s ?= %s\n", string_array_val, delete_val);
+			dI("cmp: %s ?= %s", string_array_val, delete_val);
 			if (strcmp(string_array_val, delete_val) == 0) {
-				oscap_free(string_array_val);
+				free(string_array_val);
 				string_array_val = NULL;
 				break;
 			}
@@ -1496,9 +1455,9 @@ int op_remove_strl(void *var, char *val)
 	}
 
 	newstr_array[newstr_array_size] = NULL;
-	oscap_free(string_array);
-	oscap_free(valstr_array);
-	newstr_array = oscap_realloc(newstr_array, sizeof(char*) * (newstr_array_size + 1));
+	free(string_array);
+	free(valstr_array);
+	newstr_array = realloc(newstr_array, sizeof(char*) * (newstr_array_size + 1));
 	*aptr = newstr_array;
 	return 0;
 }
@@ -1526,17 +1485,17 @@ int op_assign_disabled(void *var, char *val)
 			srv->id = strdup (tok);
 			srv->def_disabled = 1;
 
-			dI("new: def_disabled: = 1: %s\n", srv->id);
+			dI("new: def_disabled: = 1: %s", srv->id);
 
 			if (rbt_str_add (xiconf->stree, srv->id, srv) != 0) {
-				dE("Can't add service (%s) into the service tree (%p)\n",
+				dE("Can't add service (%s) into the service tree (%p)",
 				   srv->id, xiconf->stree);
 
 				xiconf_service_free(srv);
 				return (-1);
 			}
 		} else {
-			dI("mod: def_disabled: %u -> 1: %s\n", srv->def_disabled, srv->id);
+			dI("mod: def_disabled: %u -> 1: %s", srv->def_disabled, srv->id);
 			srv->def_disabled = 1;
 		}
 	}
@@ -1567,17 +1526,17 @@ int op_assign_enabled(void *var, char *val)
 			srv->id = strdup (tok);
 			srv->def_enabled = 1;
 
-			dI("new: def_enabled: = 1: %s\n", srv->id);
+			dI("new: def_enabled: = 1: %s", srv->id);
 
 			if (rbt_str_add (xiconf->stree, srv->id, srv) != 0) {
-				dE("Can't add service (%s) into the service tree (%p)\n",
+				dE("Can't add service (%s) into the service tree (%p)",
 				   srv->id, xiconf->stree);
 
 				xiconf_service_free(srv);
 				return (-1);
 			}
 		} else {
-			dI("mod: def_enabled: %u -> 1: %s\n", srv->def_disabled, srv->id);
+			dI("mod: def_enabled: %u -> 1: %s", srv->def_disabled, srv->id);
 			srv->def_enabled = 1;
 		}
 	}
@@ -1598,6 +1557,37 @@ int op_assign_enabled(void *var, char *val)
 #endif
 
 #ifndef XINETD_TEST
+static void xiservice_process_query(probe_ctx *ctx, SEXP_t *service_name, SEXP_t *protocol, const xiconf_service_t *xsrv)
+{
+	SEXP_t *xres_service_name, *xres_protocol, *item;
+
+	xres_service_name = SEXP_string_new(xsrv->name, strlen(xsrv->name));
+	xres_protocol = SEXP_string_new(xsrv->protocol, strlen(xsrv->protocol));
+	if (probe_entobj_cmp(service_name, xres_service_name) == OVAL_RESULT_TRUE &&
+	    probe_entobj_cmp(protocol, xres_protocol) == OVAL_RESULT_TRUE
+		) {
+		item = probe_item_create(OVAL_UNIX_XINETD, NULL,
+					 "protocol",         OVAL_DATATYPE_STRING,  xsrv->protocol,
+					 "service_name",     OVAL_DATATYPE_STRING,  xsrv->name,
+					 "flags",            OVAL_DATATYPE_STRING_M, xsrv->flags,
+					 "no_access",        OVAL_DATATYPE_STRING_M, xsrv->no_access,
+					 "only_from",        OVAL_DATATYPE_STRING_M, xsrv->only_from,
+					 "port",             OVAL_DATATYPE_INTEGER, (int64_t)xsrv->port,
+					 "server",           OVAL_DATATYPE_STRING,  xsrv->server,
+					 "server_arguments", OVAL_DATATYPE_STRING,  xsrv->server_args,
+					 "socket_type",      OVAL_DATATYPE_STRING,  xsrv->socket_type,
+					 "type",             OVAL_DATATYPE_STRING,  xsrv->type,
+					 "user",             OVAL_DATATYPE_STRING,  xsrv->user,
+					 "wait",             OVAL_DATATYPE_BOOLEAN, xsrv->wait,
+					 "disabled",         OVAL_DATATYPE_BOOLEAN, xsrv->disable,
+					 NULL);
+
+		probe_item_collect(ctx, item);
+	}
+	SEXP_free(xres_service_name);
+	SEXP_free(xres_protocol);
+}
+
 void *probe_init(void)
 {
 	probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, PROBE_OFFLINE_CHROOT);
@@ -1617,6 +1607,7 @@ int probe_main(probe_ctx *ctx, void *arg)
 	char    srv_prot[256];
 	int     err;
 
+	xiconf_service_t *xsrv;
 	xiconf_strans_t  *xres;
 	xiconf_t         *xcfg = (xiconf_t *)arg;
 
@@ -1675,38 +1666,17 @@ int probe_main(probe_ctx *ctx, void *arg)
 	xres = xiconf_dump(xcfg);
 
 	if (xres != NULL) {
-		SEXP_t *item, *xres_service_name, *xres_protocol;
 		register unsigned int l;
 
 		for (l = 0; l < xres->cnt; ++l) {
-			xres_service_name = SEXP_string_new( xres->srv[l]->name, strlen( xres->srv[l]->name));
-			xres_protocol = SEXP_string_new( xres->srv[l]->protocol, strlen( xres->srv[l]->protocol));
-			if (probe_entobj_cmp(service_name, xres_service_name) == OVAL_RESULT_TRUE &&
-			    probe_entobj_cmp(protocol, xres_protocol) == OVAL_RESULT_TRUE
-			   ) {
-				item = probe_item_create(OVAL_UNIX_XINETD, NULL,
-						"protocol",         OVAL_DATATYPE_STRING,  xres->srv[l]->protocol,
-						"service_name",     OVAL_DATATYPE_STRING,  xres->srv[l]->name,
-						"flags",            OVAL_DATATYPE_STRING_M, xres->srv[l]->flags,
-						"no_access",        OVAL_DATATYPE_STRING_M, xres->srv[l]->no_access,
-						"only_from",        OVAL_DATATYPE_STRING_M, xres->srv[l]->only_from,
-                                                "port",             OVAL_DATATYPE_INTEGER, (int64_t)xres->srv[l]->port,
-						"server",           OVAL_DATATYPE_STRING,  xres->srv[l]->server,
-						"server_arguments", OVAL_DATATYPE_STRING,  xres->srv[l]->server_args,
-						"socket_type",      OVAL_DATATYPE_STRING,  xres->srv[l]->socket_type,
-						"type",             OVAL_DATATYPE_STRING,  xres->srv[l]->type,
-						"user",             OVAL_DATATYPE_STRING,  xres->srv[l]->user,
-						"wait",             OVAL_DATATYPE_BOOLEAN, xres->srv[l]->wait,
-						"disabled",         OVAL_DATATYPE_BOOLEAN, xres->srv[l]->disable,
-						NULL);
-
-				probe_item_collect(ctx, item);
+			xsrv = xres->srv[l];
+			while (xsrv != NULL) {
+				xiservice_process_query(ctx, service_name, protocol, xsrv);
+				xsrv = xsrv->next;
 			}
-			SEXP_free(xres_service_name);
-			SEXP_free(xres_protocol);
 		}
-		oscap_free(xres->srv);
-		oscap_free(xres);
+		free(xres->srv);
+		free(xres);
 
 	}
 	SEXP_vfree(service_name, protocol, NULL);
