@@ -33,13 +33,7 @@
 #define XCCDF_SESSION_H_
 
 #include "xccdf_policy.h"
-
-/**
- * Type of the function used to report progress of download.
- * @param warning indicates whether the message is rather warning or notice
- * @param format printf-like format string
- */
-typedef void (*download_progress_calllback_t) (bool warning, const char * format, ...);
+#include "oscap_download_cb.h"
 
 /**
  * @struct xccdf_session
@@ -47,6 +41,20 @@ typedef void (*download_progress_calllback_t) (bool warning, const char * format
  */
 struct xccdf_session;
 
+/**
+ * Loading flags for XCCDF session
+ * @memberof xccdf_session
+ * The flags set which components will be loaded in XCCDF session.
+ */
+typedef enum {
+	XCCDF_SESSION_LOAD_NONE = 0,
+	XCCDF_SESSION_LOAD_XCCDF = 1 << 0,
+	XCCDF_SESSION_LOAD_CPE = 1 << 1,
+	XCCDF_SESSION_LOAD_OVAL = 1 << 2,
+	XCCDF_SESSION_LOAD_CHECK_ENGINE_PLUGINS = 1 << 3,
+	XCCDF_SESSION_LOAD_ALL = XCCDF_SESSION_LOAD_XCCDF | XCCDF_SESSION_LOAD_CPE | XCCDF_SESSION_LOAD_OVAL | XCCDF_SESSION_LOAD_CHECK_ENGINE_PLUGINS
+} xccdf_session_loading_flags_t;
+
 /**
  * Costructor of xccdf_session. It attempts to recognize type of the filename.
  * @memberof xccdf_session
@@ -56,6 +64,15 @@ struct xccdf_session;
  */
 struct xccdf_session *xccdf_session_new(const char *filename);
 
+/**
+ * Costructor of xccdf_session. It creates a new xccdf_session from an oscap_source structure.
+ * @memberof xccdf_session
+ * @param source oscap_source which can represent a DS or XCCDF file.
+ * @returns newly created \ref xccdf_session.
+ * @retval NULL is returned in case of error. Details might be found through \ref oscap_err_desc()
+ */
+struct xccdf_session *xccdf_session_new_from_source(struct oscap_source *source);
+
 /**
  * Destructor of xccdf_session.
  * @memberof xccdf_session
@@ -78,7 +95,19 @@ const char *xccdf_session_get_filename(const struct xccdf_session *session);
 bool xccdf_session_is_sds(const struct xccdf_session *session);
 
 /**
- * Set XSD validation level.
+ * Set rule for session - if rule is not NULL, session will use only this
+ * one rule.
+ * @memberof xccdf_session
+ * @param session XCCDF Session
+ * @param rule If not NULL, session will use only this rule
+ */
+void xccdf_session_set_rule(struct xccdf_session *session, const char *rule);
+
+/**
+ * Set XSD validation level to one of three possibilities:
+ *	- None: 	All XSD validations will be skipped.
+ *	- Default:	Partial (input) XSD validations will be done.
+ *	- Full Valid.:	Every possible (input & output) XSD validation will be done.
  * @memberof xccdf_session
  * @param session XCCDF Session
  * @param validate False value indicates to skip any XSD validation.
@@ -86,6 +115,16 @@ bool xccdf_session_is_sds(const struct xccdf_session *session);
  */
 void xccdf_session_set_validation(struct xccdf_session *session, bool validate, bool full_validation);
 
+/**
+ * Set whether the thin results override is enabled.
+ * If true the OVAL results put in ARF or separate files will have thin results.
+ * Thin results do not contain details about the evaluated criteria, only
+ * minimal OVAL results.
+ * @memberof xccdf_session
+ * @param thin_results true to enable thin_results, default is false
+ */
+void xccdf_session_set_thin_results(struct xccdf_session *session, bool thin_result);
+
 /**
  * Set requested datastream_id for this session. This datastream_id is later
  * passed down to @ref ds_sds_index_select_checklist to determine target component.
@@ -135,6 +174,12 @@ void xccdf_session_set_benchmark_id(struct xccdf_session *session, const char *b
  */
 const char *xccdf_session_get_benchmark_id(struct xccdf_session *session);
 
+/**
+ * Retrieves the result id
+ * @memberof xccdf_session
+ */
+const char *xccdf_session_get_result_id(struct xccdf_session *session);
+
 /**
  * Set path to custom CPE dictionary for the session. This function is applicable
  * only before session loads. It has no effect if run afterwards.
@@ -172,6 +217,14 @@ void xccdf_session_set_user_tailoring_cid(struct xccdf_session *session, const c
  */
 void xccdf_session_set_remote_resources(struct xccdf_session *session, bool allowed, download_progress_calllback_t callback);
 
+/**
+ * Disable or allow loading of depending content (OVAL, SCE, CPE)
+ * @memberof xccdf_session
+ * @param session XCCDF Session
+ * @param flags Bit mask that sets loading of other content in the session.
+ */
+void xccdf_session_set_loading_flags(struct xccdf_session *session, xccdf_session_loading_flags_t flags);
+
 /**
  * Set custom oval files for this session
  * @memberof xccdf_session
@@ -200,6 +253,14 @@ void xccdf_session_set_custom_oval_eval_fn(struct xccdf_session *session, xccdf_
  */
 bool xccdf_session_set_product_cpe(struct xccdf_session *session, const char *product_cpe);
 
+/**
+ * Set whether the System Characteristics shall be exported in result files.
+ * @memberof xccdf_session
+ * @param session XCCDF Session
+ * @param without_sys_chars whether to export System Characteristics or not.
+ */
+void xccdf_session_set_without_sys_chars_export(struct xccdf_session *session, bool without_sys_chars);
+
 /**
  * Set whether the OVAL result files shall be exported.
  * @memberof xccdf_session
@@ -268,6 +329,16 @@ bool xccdf_session_set_report_export(struct xccdf_session *session, const char *
  */
 bool xccdf_session_set_profile_id(struct xccdf_session *session, const char *profile_id);
 
+/**
+ *Select XCCDF Profile for evaluation with only profile suffix as input. Reports error
+ *if multiple profiles match the suffix.
+ *@memberof xccdf_session
+ *@param session XCCDF Session
+ *@param profile_suffix unique profile ID or suffix of the ID of the profile to set
+ *@returns 0 on success, 1 if profile is not found, and 2 if multiple matches are found.
+ */
+int xccdf_session_set_profile_id_by_suffix(struct xccdf_session *session, const char *profile_suffix);
+
 /**
  * Retrieves ID of the profile that we will evaluate with, or NULL.
  * @memberof xccdf_session
@@ -334,8 +405,11 @@ int xccdf_session_load_oval(struct xccdf_session *session);
  *
  * @memberof xccdf_session
  * @param session XCCDF Session
+ * @param plugin_name Name of the plugin to load
+ * @param quiet If true we will not output errors if loading fails
  * @returns zero on success
  */
+int xccdf_session_load_check_engine_plugin2(struct xccdf_session *session, const char* plugin_name, bool quiet);
 int xccdf_session_load_check_engine_plugin(struct xccdf_session *session, const char* plugin_name);
 
 /**
@@ -485,11 +559,20 @@ int xccdf_session_remediate(struct xccdf_session *session);
  * @memberof xccdf_session
  * @param session XCCDF Session
  * @param testresult_id ID of the TestResult element in the file (the NULL value stands
- * for the last TestResult).
+ * for the last TestResult). Suffix match is attempted if exact match is not found.
  * @returns zero on success.
  */
 int xccdf_session_build_policy_from_testresult(struct xccdf_session *session, const char *testresult_id);
 
+/**
+ * Load xccdf:TestResult to the session from oscap_source
+ * @memberof xccdf_session
+ * @param session XCCDF Session
+ * @param report_source Structure conataining oscap_source of the test results
+ * @returns zero on success.
+ */
+int xccdf_session_add_report_from_source(struct xccdf_session *session, struct oscap_source *report_source);
+
 /// @}
 /// @}
 #endif
