@@ -74,7 +74,7 @@ struct xccdf_result *xccdf_result_new(void)
 
 struct xccdf_result * xccdf_result_clone(const struct xccdf_result * result)
 {
-	struct xccdf_item *new_result = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_result_item));
+	struct xccdf_item *new_result = calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_result_item));
 	struct xccdf_item *old = XITEM(result);
     xccdf_item_base_clone(&new_result->item, &(old->item));
 	new_result->type = old->type;
@@ -85,23 +85,23 @@ struct xccdf_result * xccdf_result_clone(const struct xccdf_result * result)
 static inline void xccdf_result_free_impl(struct xccdf_item *result)
 {
 	if (result != NULL) {
-		oscap_free(result->sub.result.start_time);
-		oscap_free(result->sub.result.end_time);
-		oscap_free(result->sub.result.test_system);
-		oscap_free(result->sub.result.benchmark_uri);
-		oscap_free(result->sub.result.profile);
+		free(result->sub.result.start_time);
+		free(result->sub.result.end_time);
+		free(result->sub.result.test_system);
+		free(result->sub.result.benchmark_uri);
+		free(result->sub.result.profile);
 
 		oscap_list_free(result->sub.result.identities, (oscap_destruct_func) xccdf_identity_free);
 		oscap_list_free(result->sub.result.target_facts, (oscap_destruct_func) xccdf_target_fact_free);
 		oscap_list_free(result->sub.result.target_id_refs, (oscap_destruct_func) xccdf_target_identifier_free);
-		oscap_list_free(result->sub.result.applicable_platforms, oscap_free);
-		oscap_list_free(result->sub.result.targets, oscap_free);
+		oscap_list_free(result->sub.result.applicable_platforms, free);
+		oscap_list_free(result->sub.result.targets, free);
 		oscap_list_free(result->sub.result.scores, (oscap_destruct_func) xccdf_score_free);
 		oscap_list_free(result->sub.result.remarks, (oscap_destruct_func) oscap_text_free);
-		oscap_list_free(result->sub.result.target_addresses, oscap_free);
+		oscap_list_free(result->sub.result.target_addresses, free);
 		oscap_list_free(result->sub.result.setvalues, (oscap_destruct_func) xccdf_setvalue_free);
 		oscap_list_free(result->sub.result.rule_results, (oscap_destruct_func) xccdf_rule_result_free);
-		oscap_list_free(result->sub.result.organizations, oscap_free);
+		oscap_list_free(result->sub.result.organizations, free);
 
 		xccdf_item_release(result);
 	}
@@ -156,8 +156,8 @@ static inline void _xccdf_result_fill_identity(struct xccdf_result *result)
 
 static inline void _xccdf_result_clear_metadata(struct xccdf_item *result)
 {
-	oscap_list_free(result->sub.result.targets, oscap_free);
-	oscap_list_free(result->sub.result.target_addresses, oscap_free);
+	oscap_list_free(result->sub.result.targets, free);
+	oscap_list_free(result->sub.result.target_addresses, free);
 	oscap_list_free(result->sub.result.target_facts, (oscap_destruct_func) xccdf_target_fact_free);
 	oscap_list_free(result->sub.result.identities, (oscap_destruct_func) xccdf_identity_free);
 	oscap_create_lists(
@@ -247,7 +247,7 @@ void xccdf_result_fill_sysinfo(struct xccdf_result *result)
 
 struct xccdf_rule_result *xccdf_rule_result_new(void)
 {
-	struct xccdf_rule_result *rr = oscap_calloc(1, sizeof(struct xccdf_rule_result));
+	struct xccdf_rule_result *rr = calloc(1, sizeof(struct xccdf_rule_result));
 	oscap_create_lists(&rr->overrides, &rr->idents, &rr->messages,
 		&rr->instances, &rr->fixes, &rr->checks, NULL);
 	return rr;
@@ -256,9 +256,9 @@ struct xccdf_rule_result *xccdf_rule_result_new(void)
 void xccdf_rule_result_free(struct xccdf_rule_result *rr)
 {
 	if (rr != NULL) {
-		oscap_free(rr->idref);
-		oscap_free(rr->version);
-		oscap_free(rr->time);
+		free(rr->idref);
+		free(rr->version);
+		free(rr->time);
 
 		oscap_list_free(rr->overrides, (oscap_destruct_func) xccdf_override_free);
 		oscap_list_free(rr->idents, (oscap_destruct_func) xccdf_ident_free);
@@ -267,7 +267,7 @@ void xccdf_rule_result_free(struct xccdf_rule_result *rr)
 		oscap_list_free(rr->fixes, (oscap_destruct_func) xccdf_fix_free);
 		oscap_list_free(rr->checks, (oscap_destruct_func) xccdf_check_free);
 
-		oscap_free(rr);
+		free(rr);
 	}
 }
 
@@ -293,14 +293,14 @@ OSCAP_ITERATOR_REMOVE_F(xccdf_rule_result)
 
 struct xccdf_identity *xccdf_identity_new(void)
 {
-	return oscap_calloc(1, sizeof(struct xccdf_identity));
+	return calloc(1, sizeof(struct xccdf_identity));
 }
 
 void xccdf_identity_free(struct xccdf_identity *identity)
 {
 	if (identity != NULL) {
-		oscap_free(identity->name);
-		oscap_free(identity);
+		free(identity->name);
+		free(identity);
 	}
 }
 
@@ -312,7 +312,7 @@ OSCAP_ITERATOR_REMOVE_F(xccdf_identity)
 
 struct xccdf_score *xccdf_score_new(void)
 {
-	struct xccdf_score *score = oscap_calloc(1, sizeof(struct xccdf_score));
+	struct xccdf_score *score = calloc(1, sizeof(struct xccdf_score));
 	score->score = NAN;
 	score->maximum = XCCDF_SCORE_MAX_DAFAULT;
 	return score;
@@ -321,8 +321,8 @@ struct xccdf_score *xccdf_score_new(void)
 void xccdf_score_free(struct xccdf_score *score)
 {
 	if (score != NULL) {
-		oscap_free(score->system);
-		oscap_free(score);
+		free(score->system);
+		free(score);
 	}
 }
 
@@ -334,15 +334,15 @@ OSCAP_ITERATOR_REMOVE_F(xccdf_score)
 
 struct xccdf_override *xccdf_override_new(void)
 {
-	return oscap_calloc(1, sizeof(struct xccdf_override));
+	return calloc(1, sizeof(struct xccdf_override));
 }
 
 void xccdf_override_free(struct xccdf_override *oride)
 {
 	if (oride != NULL) {
-		oscap_free(oride->authority);
+		free(oride->authority);
 		oscap_text_free(oride->remark);
-		oscap_free(oride);
+		free(oride);
 	}
 }
 
@@ -354,14 +354,14 @@ OSCAP_ACCESSOR_TEXT(xccdf_override, remark)
 
 struct xccdf_message *xccdf_message_new(void)
 {
-    return oscap_calloc(1, sizeof(struct xccdf_message));
+    return calloc(1, sizeof(struct xccdf_message));
 }
 
 void xccdf_message_free(struct xccdf_message *msg)
 {
     if (msg != NULL) {
-        oscap_free(msg->content);
-        oscap_free(msg);
+        free(msg->content);
+        free(msg);
     }
 }
 
@@ -370,22 +370,22 @@ OSCAP_ACCESSOR_STRING(xccdf_message, content)
 
 struct xccdf_target_fact* xccdf_target_fact_new(void)
 {
-    return oscap_calloc(1, sizeof(struct xccdf_target_fact));
+    return calloc(1, sizeof(struct xccdf_target_fact));
 }
 
 void xccdf_target_fact_free(struct xccdf_target_fact *fact)
 {
     if (fact != NULL) {
-        oscap_free(fact->name);
-        oscap_free(fact->value);
-        oscap_free(fact);
+        free(fact->name);
+        free(fact->value);
+        free(fact);
     }
 }
 
 static inline bool xccdf_target_fact_set_value(struct xccdf_target_fact *fact, xccdf_value_type_t type, const char *str)
 {
     assert(fact != NULL);
-    oscap_free(fact->value);
+    free(fact->value);
     fact->value = oscap_strdup(str);
     fact->type  = type;
     return true;
@@ -416,7 +416,7 @@ OSCAP_ITERATOR_REMOVE_F(xccdf_target_fact)
 
 struct xccdf_target_identifier* xccdf_target_identifier_new(void)
 {
-    return oscap_calloc(1, sizeof(struct xccdf_target_identifier));
+    return calloc(1, sizeof(struct xccdf_target_identifier));
 }
 
 struct xccdf_target_identifier* xccdf_target_identifier_clone(const struct xccdf_target_identifier* ti)
@@ -442,21 +442,21 @@ void xccdf_target_identifier_free(struct xccdf_target_identifier *ti)
     		xmlFreeNode(ti->element);
     	}
 		else {
-        	oscap_free(ti->system);
-        	oscap_free(ti->href);
-        	oscap_free(ti->name);
+			free(ti->system);
+			free(ti->href);
+			free(ti->name);
         }
 
-    	oscap_free(ti);
+		free(ti);
     }
 }
 
 bool xccdf_target_identifier_set_xml_node(struct xccdf_target_identifier *ti, void* node)
 {
 	if (!ti->any_element) {
-		oscap_free(ti->system);
-		oscap_free(ti->href);
-		oscap_free(ti->name);
+		free(ti->system);
+		free(ti->href);
+		free(ti->name);
 	}
 	else if (ti->element)
 		xmlFreeNode(ti->element);
@@ -483,7 +483,7 @@ bool xccdf_target_identifier_set_system(struct xccdf_target_identifier *ti, cons
 	}
 
 	ti->any_element = false;
-	oscap_free(ti->system);
+	free(ti->system);
 	ti->system = oscap_strdup(newval);
 
 	return true;
@@ -505,7 +505,7 @@ bool xccdf_target_identifier_set_href(struct xccdf_target_identifier *ti, const
 	}
 
 	ti->any_element = false;
-	oscap_free(ti->href);
+	free(ti->href);
 	ti->href = oscap_strdup(newval);
 
 	return true;
@@ -527,7 +527,7 @@ bool xccdf_target_identifier_set_name(struct xccdf_target_identifier *ti, const
 	}
 
 	ti->any_element = false;
-	oscap_free(ti->name);
+	free(ti->name);
 	ti->name = oscap_strdup(newval);
 
 	return true;
@@ -546,7 +546,7 @@ OSCAP_ITERATOR_REMOVE_F(xccdf_target_identifier)
 
 struct xccdf_instance *xccdf_instance_new(void)
 {
-    struct xccdf_instance *inst = oscap_calloc(1, sizeof(struct xccdf_instance));
+    struct xccdf_instance *inst = calloc(1, sizeof(struct xccdf_instance));
     inst->context = oscap_strdup(XCCDF_INSTANCE_DEFAULT_CONTEXT);
     return inst;
 }
@@ -554,10 +554,10 @@ struct xccdf_instance *xccdf_instance_new(void)
 void xccdf_instance_free(struct xccdf_instance *inst)
 {
     if (inst != NULL) {
-        oscap_free(inst->context);
-        oscap_free(inst->parent_context);
-        oscap_free(inst->content);
-        oscap_free(inst);
+        free(inst->context);
+        free(inst->parent_context);
+        free(inst->content);
+        free(inst);
     }
 }
 
@@ -799,6 +799,10 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 	const char *version = xccdf_result_get_version(result);
         if (version != NULL)
 		xmlNewProp(result_node, BAD_CAST "version", BAD_CAST version);
+	const char *test_system = xccdf_result_get_test_system(result);
+	if (test_system != NULL) {
+		xmlNewProp(result_node, BAD_CAST "test-system", BAD_CAST test_system);
+	}
 
 	/* Handle children */
 	xccdf_texts_to_dom(xccdf_result_get_remarks(result), result_node, "remark");
@@ -851,7 +855,7 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 			// expanded IPv6 addresses.
 
 			xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "target-address", BAD_CAST expanded_ipv6);
-			oscap_free(expanded_ipv6);
+			free(expanded_ipv6);
 		}
 		else {
 			xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "target-address", BAD_CAST target_address);
@@ -914,7 +918,7 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
                 float value = xccdf_score_get_score(score);
                 char *value_str = oscap_sprintf("%f", value);
                 xmlNode *score_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "score", BAD_CAST value_str);
-                oscap_free(value_str);
+                free(value_str);
 
 		const char *sys = xccdf_score_get_system(score);
 		if (sys)
@@ -923,7 +927,7 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 		float max = xccdf_score_get_maximum(score);
 		char *max_str = oscap_sprintf("%f", max);
 		xmlNewProp(score_node, BAD_CAST "maximum", BAD_CAST max_str);
-        oscap_free(max_str);
+        free(max_str);
 	}
 	xccdf_score_iterator_free(scores);
 }
@@ -1088,7 +1092,7 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 	float weight = xccdf_rule_result_get_weight(result);
 	char *weight_str = oscap_sprintf("%f", weight);
 	xmlNewProp(result_node, BAD_CAST "weight", BAD_CAST weight_str);
-    oscap_free(weight_str);
+    free(weight_str);
 
 	/* Handle children */
 	xccdf_test_result_type_t test_res = xccdf_rule_result_get_result(result);
@@ -1113,7 +1117,9 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 	while (xccdf_message_iterator_has_more(messages)) {
 		struct xccdf_message *message = xccdf_message_iterator_next(messages);
 		const char *content = xccdf_message_get_content(message);
-		xmlNode *message_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "message", BAD_CAST content);
+		xmlChar *encoded_content = xmlEncodeEntitiesReentrant(doc, BAD_CAST content);
+		xmlNode *message_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "message", encoded_content);
+		xmlFree(encoded_content);
 
                 xccdf_level_t message_severity = xccdf_message_get_severity(message);
 		if (message_severity != XCCDF_LEVEL_NOT_DEFINED)
