@@ -137,7 +137,7 @@ char *xccdf_policy_get_readable_item_title(struct xccdf_policy *policy, struct x
 	if (!unresolved)
 		return oscap_strdup("");
 	char *resolved = xccdf_policy_substitute(unresolved, policy);
-	oscap_free(unresolved);
+	free(unresolved);
 	return resolved;
 }
 
@@ -151,10 +151,10 @@ char *xccdf_policy_get_readable_item_description(struct xccdf_policy *policy, st
 		return oscap_strdup("");
 	const char *unresolved = oscap_text_get_text(unresolved_text);
 	/* Resolve <xccdf:sub> elements */
-	const char *resolved = xccdf_policy_substitute(unresolved, policy);
+	char *resolved = xccdf_policy_substitute(unresolved, policy);
 	/* Get a rid of xhtml elements */
 	char *plaintext = _xhtml_to_plaintext(resolved);
-	oscap_free(resolved);
+	free(resolved);
 	return plaintext;
 }
 
@@ -437,7 +437,7 @@ static struct oscap_list * xccdf_policy_check_get_value_bindings(struct xccdf_po
             value = (struct xccdf_value *) xccdf_benchmark_get_item(benchmark, xccdf_check_export_get_value(check));
             if (value == NULL) {
                 oscap_seterr(OSCAP_EFAMILY_XCCDF, "Value \"%s\" does not exist in benchmark", xccdf_check_export_get_value(check));
-		oscap_list_free(list, oscap_free);
+		oscap_list_free(list, free);
                 return NULL;
             }
             binding = xccdf_value_binding_new();
@@ -461,7 +461,7 @@ static struct oscap_list * xccdf_policy_check_get_value_bindings(struct xccdf_po
             const struct xccdf_value_instance * val = xccdf_value_get_instance_by_selector(value, selector);
             if (val == NULL) {
                 oscap_seterr(OSCAP_EFAMILY_XCCDF, "Attempt to get non-existent selector \"%s\" from variable \"%s\"", selector, xccdf_value_get_id(value));
-		oscap_list_free(list, oscap_free);
+		oscap_list_free(list, free);
 		xccdf_value_binding_free(binding);
                 return NULL;
             }
@@ -657,24 +657,8 @@ static struct xccdf_rule_result * _xccdf_rule_result_new_from_rule(const struct
 		xccdf_rule_result_add_ident(rule_ritem, xccdf_ident_clone(ident));
 	}
 	xccdf_ident_iterator_free(ident_it);
-	if (check != NULL) {
+	if (check != NULL)
 		xccdf_rule_result_add_check(rule_ritem, check);
-	} else {
-		/* Workaround: No applicable or candidate check was found. However,
-		 * the ARF schematron requires us to include at least one check.
-		 *
-		 * For more info see thread on xccdf-dev mailing list named:
-		 *     [Xccdf-dev] xccdf:rule-result element properties
-		 */
-		struct xccdf_check_iterator *check_it = xccdf_rule_get_checks(rule);
-		while (xccdf_check_iterator_has_more(check_it)) {
-			struct xccdf_check *orig_check = xccdf_check_iterator_next(check_it);
-			struct xccdf_check *new_check = xccdf_check_clone(orig_check);
-			xccdf_rule_result_add_check(rule_ritem, new_check);
-		}
-		xccdf_check_iterator_free(check_it);
-
-	}
 
 	if (message != NULL) {
 		struct xccdf_message *msg = xccdf_message_new();
@@ -706,6 +690,13 @@ static int _xccdf_policy_report_rule_result(struct xccdf_policy *policy,
 	} else
 		xccdf_check_free(check);
 
+	/* If policy selects only one rule, skip reporting for the other
+	 * unselected rules - only the selected rule will be reported. */
+	if (policy->rule != NULL) {
+		const char* rule_id = xccdf_rule_get_id(rule);
+		if (strcmp(policy->rule, rule_id) != 0)
+			return ret;
+	}
 	ret = xccdf_policy_report_cb(policy, XCCDF_POLICY_OUTCB_END, (void *) rule_result);
 	return ret;
 }
@@ -754,7 +745,7 @@ static char *_cpe_get_oval_href(struct cpe_dict_model *dict, struct cpe_lang_mod
 		} else {
 			oval_href = oscap_sprintf("%s/%s", prefix_dirname, oval_relative_href);
 		}
-		oscap_free(origin_file);
+		free(origin_file);
 	}
 	return oval_href;
 }
@@ -769,7 +760,7 @@ static bool _xccdf_policy_cpe_check_cb(const char* sys, const char* href, const
 
 	char* prefixed_href = _cpe_get_oval_href(cb_usr->dict, cb_usr->lang_model, href);
 	struct oval_agent_session *session = cpe_session_lookup_oval_session(model->cpe, prefixed_href);
-	oscap_free(prefixed_href);
+	free(prefixed_href);
 	if (session == NULL) {
 		return false;
 	}
@@ -827,12 +818,12 @@ static bool xccdf_policy_model_platforms_are_applicable_dict(struct xccdf_policy
 
 		struct cpe_name* name = cpe_name_new(platform);
 
-		struct cpe_check_cb_usr* usr = oscap_alloc(sizeof(struct cpe_check_cb_usr));
+		struct cpe_check_cb_usr* usr = malloc(sizeof(struct cpe_check_cb_usr));
 		usr->model = model;
 		usr->dict = dict;
 		usr->lang_model = NULL;
 		const bool applicable = cpe_name_applicable_dict(name, dict, (cpe_check_fn) _xccdf_policy_cpe_check_cb, usr);
-		oscap_free(usr);
+		free(usr);
 
 		cpe_name_free(name);
 
@@ -870,12 +861,12 @@ static bool xccdf_policy_model_platforms_are_applicable_lang_model(struct xccdf_
 			platform_shifted++;
 		}
 
-		struct cpe_check_cb_usr* usr = oscap_alloc(sizeof(struct cpe_check_cb_usr));
+		struct cpe_check_cb_usr* usr = malloc(sizeof(struct cpe_check_cb_usr));
 		usr->model = model;
 		usr->dict = NULL;
 		usr->lang_model = lang_model;
 		const bool applicable = cpe_platform_applicable_lang_model(platform_shifted, lang_model, (cpe_check_fn)_xccdf_policy_cpe_check_cb, (cpe_dict_fn)_xccdf_policy_cpe_dict_cb, usr);
-		oscap_free(usr);
+		free(usr);
 
 		if (applicable)
 		{
@@ -966,14 +957,25 @@ _xccdf_policy_rule_evaluate(struct xccdf_policy * policy, const struct xccdf_rul
 	const char* rule_id = xccdf_rule_get_id(rule);
 	const bool is_selected = xccdf_policy_is_item_selected(policy, rule_id);
 	const char *message = NULL;
+	int report = 0;
 
-	int report = xccdf_policy_report_cb(policy, XCCDF_POLICY_OUTCB_START, (void *) rule);
+	/* If policy selects only one rule and the rule currently being
+	 * evaluated is not equal to the selected rule, do not evaluate it and
+	 * mark it as notselected. */
+	if (policy->rule != NULL) {
+		if (strcmp(policy->rule, rule_id) != 0) {
+			return _xccdf_policy_report_rule_result(policy, result, rule, NULL, XCCDF_RESULT_NOT_SELECTED, NULL);
+		}
+		policy->rule_found = 1;
+	}
+	/* Otherwise start reporting */
+	report = xccdf_policy_report_cb(policy, XCCDF_POLICY_OUTCB_START, (void *) rule);
 	if (report)
 		return report;
 
 	struct xccdf_refine_rule_internal* r_rule = oscap_htable_get(policy->refine_rules_internal, rule_id);
-
 	xccdf_role_t role = xccdf_get_final_role(rule, r_rule);
+
 	if (!is_selected) {
 		dI("Rule '%s' is not selected.", rule_id);
 		return _xccdf_policy_report_rule_result(policy, result, rule, NULL, XCCDF_RESULT_NOT_SELECTED, NULL);
@@ -1079,6 +1081,10 @@ _xccdf_policy_rule_evaluate(struct xccdf_policy * policy, const struct xccdf_rul
 	}
 	if ((xccdf_test_result_type_t) ret == XCCDF_RESULT_NOT_CHECKED)
 		message = "None of the check-content-ref elements was resolvable.";
+
+	if (role == XCCDF_ROLE_UNSCORED)
+		ret = XCCDF_RESULT_INFORMATIONAL;
+
 	xccdf_check_content_ref_iterator_free(content_it);
 	oscap_list_free(bindings, (oscap_destruct_func) xccdf_value_binding_free);
 	/* Negate only once */
@@ -1136,7 +1142,7 @@ struct oscap_file_entry {
 
 struct oscap_file_entry *oscap_file_entry_new(void)
 {
-	struct oscap_file_entry *ret = oscap_calloc(1, sizeof(struct oscap_file_entry));
+	struct oscap_file_entry *ret = calloc(1, sizeof(struct oscap_file_entry));
 	return ret;
 }
 
@@ -1153,9 +1159,9 @@ struct oscap_file_entry *oscap_file_entry_dup(struct oscap_file_entry * file_ent
 
 void oscap_file_entry_free(struct oscap_file_entry * entry)
 {
-	oscap_free(entry->system_name);
-	oscap_free(entry->file);
-	oscap_free(entry);
+	free(entry->system_name);
+	free(entry->file);
+	free(entry);
 }
 
 const char* oscap_file_entry_get_system(struct oscap_file_entry* entry)
@@ -1460,7 +1466,8 @@ bool xccdf_policy_model_set_tailoring(struct xccdf_policy_model *model, struct x
 	xccdf_tailoring_free(model->tailoring);
 	model->tailoring = tailoring;
 
-	xccdf_tailoring_resolve(tailoring, xccdf_policy_model_get_benchmark(model));
+	if (tailoring)
+		xccdf_tailoring_resolve(tailoring, xccdf_policy_model_get_benchmark(model));
 
 	return true;
 }
@@ -1567,14 +1574,14 @@ void xccdf_policy_model_unregister_engines(struct xccdf_policy_model *model, con
 {
 	__attribute__nonnull__(model);
 	if (sys == NULL)
-		oscap_list_free(model->engines, (oscap_destruct_func) oscap_free);
+		oscap_list_free(model->engines, (oscap_destruct_func) free);
 	else {
 		struct oscap_list *rest = oscap_list_new();
 		struct oscap_iterator *cb_it = oscap_iterator_new(model->engines);
 		while (oscap_iterator_has_more(cb_it)) {
 			struct xccdf_policy_engine *engine = oscap_iterator_next(cb_it);
 			if (xccdf_policy_engine_filter(engine, sys))
-				oscap_free(engine);
+				free(engine);
 			else
 				oscap_list_add(rest, engine);
 		}
@@ -1634,7 +1641,7 @@ struct xccdf_policy_model * xccdf_policy_model_new(struct xccdf_benchmark * benc
 
 	struct xccdf_policy_model       * model;
 
-	model = oscap_alloc(sizeof(struct xccdf_policy_model));
+	model = malloc(sizeof(struct xccdf_policy_model));
 	if (model == NULL)
 		return NULL;
 	memset(model, 0, sizeof(struct xccdf_policy_model));
@@ -1677,7 +1684,7 @@ _xccdf_policy_add_selector_internal(struct xccdf_policy *policy, struct xccdf_be
 	/** If the selector doesn't point to an item it can still point to the cluster. */
 	struct oscap_htable_iterator *hit = xccdf_benchmark_get_cluster_items(benchmark, xccdf_select_get_item(sel));
 	if (hit == NULL) {
-		oscap_seterr(OSCAP_EFAMILY_XCCDF, "Selector ID(%s) does not exist in Benchmark.", xccdf_select_get_item(sel));
+		dW("Selector ID(%s) does not exist in Benchmark and it will be ignored.", xccdf_select_get_item(sel));
 		return false;
 	}
 	if (!oscap_htable_iterator_has_more(hit)) {
@@ -1769,7 +1776,7 @@ struct xccdf_policy * xccdf_policy_new(struct xccdf_policy_model * model, struct
 	struct xccdf_item_iterator      * item_it;
 	struct xccdf_item               * item;
 
-	policy = oscap_alloc(sizeof(struct xccdf_policy));
+	policy = malloc(sizeof(struct xccdf_policy));
 	if (policy == NULL)
 		return NULL;
 	memset(policy, 0, sizeof(struct xccdf_policy));
@@ -1809,7 +1816,7 @@ struct xccdf_value_binding * xccdf_value_binding_new() {
         struct xccdf_value_binding          * binding;
 
         /* Initialization */
-	binding = oscap_alloc(sizeof(struct xccdf_value_binding));
+	binding = malloc(sizeof(struct xccdf_value_binding));
 	if (binding == NULL)
 		return NULL;
 	memset(binding, 0, sizeof(struct xccdf_value_binding));
@@ -1992,9 +1999,10 @@ struct xccdf_result * xccdf_policy_evaluate(struct xccdf_policy * policy)
     struct xccdf_result * result = xccdf_result_new();
 
 	xccdf_result_set_start_time_current(result);
+	xccdf_result_set_test_system(result, "cpe:/a:redhat:openscap:" VERSION);
 
     /** Set ID of TestResult */
-    const char * id = NULL;
+	char *id = NULL;
 	if ((xccdf_policy_get_profile(policy) != NULL) && (xccdf_profile_get_id(xccdf_policy_get_profile(policy)) != NULL)) {
 		id = oscap_strdup(xccdf_profile_get_id(xccdf_policy_get_profile(policy)));
 		xccdf_result_set_profile(result, id);
@@ -2030,7 +2038,7 @@ struct xccdf_result * xccdf_policy_evaluate(struct xccdf_policy * policy)
         xccdf_result_set_id(result, rid);
     }
 
-    oscap_free(id);
+    free(id);
 
 	/** We need to process document top-down order.
 	 * See conflicts/requires and Item Processing Algorithm */
@@ -2048,6 +2056,12 @@ struct xccdf_result * xccdf_policy_evaluate(struct xccdf_policy * policy)
 	}
 	xccdf_item_iterator_free(item_it);
 
+	if (policy->rule != NULL && !policy->rule_found) {
+		oscap_seterr(OSCAP_EFAMILY_XCCDF,
+			"Rule '%s' not found in selected profile.", policy->rule);
+		return NULL;
+	}
+
 	xccdf_policy_add_final_setvalues(policy, xccdf_benchmark_to_item(benchmark), result);
 
 	struct oscap_htable_iterator *it = oscap_htable_iterator_new(policy->model->cpe->applicable_platforms);
@@ -2070,6 +2084,11 @@ struct xccdf_score * xccdf_policy_get_score(struct xccdf_policy * policy, struct
 	return xccdf_result_calculate_score(test_result, (struct xccdf_item *) benchmark, scsystem);
 }
 
+int xccdf_policy_recalculate_score(struct xccdf_policy * policy, struct xccdf_result * test_result)
+{
+	struct xccdf_benchmark * benchmark = xccdf_policy_model_get_benchmark(xccdf_policy_get_model(policy));
+	return xccdf_result_recalculate_scores(test_result, (struct xccdf_item *) benchmark);
+}
 
 const char *xccdf_policy_get_value_of_item(struct xccdf_policy * policy, struct xccdf_item * item)
 {
@@ -2150,8 +2169,8 @@ struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struc
                 xccdf_rule_set_severity((struct xccdf_rule *) new_item, xccdf_refine_rule_internal_get_severity(r_rule));
             if (xccdf_weight_defined(xccdf_refine_rule_internal_get_weight(r_rule)))
                 xccdf_rule_set_weight((struct xccdf_rule *) new_item, xccdf_refine_rule_internal_get_weight(r_rule));
-                break;
-            }
+            break;
+        }
         case XCCDF_GROUP: {
         struct xccdf_refine_rule_internal * r_rule = xccdf_policy_get_refine_rule_by_item(policy, item);
             if (r_rule == NULL) return item;
@@ -2228,23 +2247,26 @@ void xccdf_policy_model_free(struct xccdf_policy_model * model) {
 
 	oscap_list_free(model->policies, (oscap_destruct_func) xccdf_policy_free);
 	xccdf_policy_model_unregister_engines(model, NULL);
-	oscap_list_free(model->callbacks, (oscap_destruct_func) oscap_free);
+	oscap_list_free(model->callbacks, (oscap_destruct_func) free);
 	xccdf_tailoring_free(model->tailoring);
         xccdf_benchmark_free(model->benchmark);
 	cpe_session_free(model->cpe);
-        oscap_free(model);
+        free(model);
 }
 
 
 
 void xccdf_policy_free(struct xccdf_policy * policy) {
 
-        /* If ID of policy's profile is NULL then this
-         * profile is created by Policy layer and need
-         * to be freed
-         */
-        if (xccdf_profile_get_id(policy->profile) == NULL)
-            xccdf_profile_free((struct xccdf_item *) policy->profile);
+	/* A policy which is set to use default profile has its profile member set to NULL,
+	 * check it so we don't try to get the ID from a NULL profile.
+	 * */
+	if (policy->profile && xccdf_profile_get_id(policy->profile) == NULL)
+		/* If ID of policy's profile is NULL then this
+		 * profile is created by Policy layer and need
+		 * to be freed
+		 */
+		xccdf_profile_free((struct xccdf_item *) policy->profile);
 
 	oscap_list_free(policy->selects, (oscap_destruct_func) xccdf_select_free);
 	oscap_list_free(policy->values, (oscap_destruct_func) xccdf_value_binding_free);
@@ -2252,15 +2274,15 @@ void xccdf_policy_free(struct xccdf_policy * policy) {
 	oscap_htable_free0(policy->selected_internal);
 	oscap_htable_free0(policy->selected_final);
 	oscap_htable_free(policy->refine_rules_internal, (oscap_destruct_func) xccdf_refine_rule_internal_free);
-        oscap_free(policy);
+        free(policy);
 }
 
 void xccdf_value_binding_free(struct xccdf_value_binding * binding) {
 
-        oscap_free(binding->name);
-        oscap_free(binding->value);
-        oscap_free(binding->setvalue);
-        oscap_free(binding);
+        free(binding->name);
+        free(binding->value);
+        free(binding->setvalue);
+        free(binding);
 }
 
 
