@@ -52,7 +52,7 @@ static int _rule_add_info_message(struct xccdf_rule_result *rr, ...)
 
 	msg = xccdf_message_new();
 	assume_ex(xccdf_message_set_content(msg, text), 1);
-	oscap_free(text);
+	free(text);
 	assume_ex(xccdf_message_set_severity(msg, XCCDF_MSG_INFO), 1);
 	assume_ex(xccdf_rule_result_add_message(rr, msg), 1);
 	return 0;
@@ -64,20 +64,77 @@ static inline bool _file_exists(const char *file)
 	return file != NULL && stat(file, &sb) == 0;
 }
 
-static int _write_text_to_fd_and_free(int output_fd, const char *text)
-{
-	ssize_t len = strlen(text);
+static int _write_text_to_fd(int output_fd, const char* text) {
+
 	ssize_t written = 0;
-	while (written < len) {
-		ssize_t w = write(output_fd, text + written, len - written);
+	ssize_t length = strlen(text);
+
+	while (written < length) {
+		ssize_t w = write(output_fd, text + written, length - written);
 		if (w < 0)
 			break;
 		written += w;
 	}
-	oscap_free(text);
-	return written != len;
+
+	return written != length;
+
 }
 
+static int _write_text_to_fd_and_free(int output_fd, char *text)
+{
+	int ret = _write_text_to_fd(output_fd, text);
+	free(text);
+	return ret;
+}
+
+static int _write_remediation_to_fd_and_free(int output_fd, const char* template, char* text)
+{
+	if (oscap_streq(template, "urn:xccdf:fix:script:ansible")) {
+		// Add required indentation in front of every single line
+
+		const char delim = '\n';
+		const char *indentation = "    ";
+
+		char *current = text;
+		char *next_delim = NULL;
+
+		do {
+			next_delim = strchr(current, delim);
+			if (next_delim != NULL) {
+				*next_delim = '\0';
+			}
+
+			// write indentation
+			if (_write_text_to_fd(output_fd, indentation) != 0) {
+				free(text);
+				return 1;
+			}
+
+			// write rest of line
+			if (_write_text_to_fd(output_fd, current) != 0) {
+				free(text);
+				return 1;
+			}
+			if (_write_text_to_fd(output_fd, "\n") != 0) {
+				free(text);
+				return 1;
+			}
+
+			if (next_delim != NULL) {
+				// text is NULL terminated to this is guaranteed to point to valid memory
+				current = next_delim + 1;
+			}
+		}
+		while (next_delim != NULL);
+
+		free(text);
+		return 0;
+
+	} else {
+		// no extra processing is needed
+		return _write_text_to_fd_and_free(output_fd, text);
+	}
+}
 struct _interpret_map {
 	const char *sys;
 	const char *interpret;
@@ -104,7 +161,10 @@ static const char *_get_supported_interpret(const char *sys, const struct _inter
 		{"urn:xccdf:fix:script:csh",		"/bin/csh"},
 		{"urn:xccdf:fix:script:tclsh",		"/usr/bin/tclsh"},
 		{"urn:xccdf:fix:script:javascript",	"/usr/bin/js"},
-		{"urn:xccdf:fix:script:ansible",	"/usr/bin/ansible-playbook"},
+
+		// Current Ansible remediations are only Ansible snippets and are
+		// not runnable without header.
+		// {"urn:xccdf:fix:script:ansible",	"/usr/bin/ansible-playbook"},
 		{NULL,					NULL}
 	};
 	const char *interpret = _search_interpret_map(sys, _openscap_supported_interprets);
@@ -132,6 +192,19 @@ static inline bool _is_platform_applicable(struct xccdf_policy *policy, const ch
 	return ret;
 }
 
+static struct oscap_list *_get_fixes(struct xccdf_policy *policy, const struct xccdf_rule *rule)
+{
+	struct oscap_list *result = oscap_list_new();
+
+	struct xccdf_fix_iterator *fix_it = xccdf_rule_get_fixes(rule);
+	while (xccdf_fix_iterator_has_more(fix_it)) {
+		struct xccdf_fix *fix = xccdf_fix_iterator_next(fix_it);
+		oscap_list_add(result, fix);
+	}
+	xccdf_fix_iterator_free(fix_it);
+	return result;
+}
+
 static struct oscap_list *_filter_fixes_by_applicability(struct xccdf_policy *policy, const struct xccdf_rule *rule)
 {
 	/* Filters out the fixes which are not applicable */
@@ -237,7 +310,7 @@ static inline int _xccdf_fix_decode_xml(struct xccdf_fix *fix, char **result)
         xmlDoc *doc = xmlReadMemory(str, strlen(str), NULL, NULL, XML_PARSE_RECOVER |
 		XML_PARSE_NOERROR | XML_PARSE_NOWARNING | XML_PARSE_NONET | XML_PARSE_NSCLEAN);
 	dI("Following script will be executed: '''%s'''", str);
-	oscap_free(str);
+	free(str);
 
         xmlBuffer *buff = xmlBufferCreate();
 	xmlNodePtr child = xmlDocGetRootElement(doc)->children;
@@ -343,21 +416,21 @@ static inline int _xccdf_fix_execute(struct xccdf_rule_result *rr, struct xccdf_
 			printf("Error while executing fix script: execve returned: %s\n", strerror(errno));
 			exit(42);
 		} else {
-			oscap_free(temp_file);
+			free(temp_file);
 			close(pipefd[1]);
-			const char *stdout_buff = oscap_acquire_pipe_to_string(pipefd[0]);
+			char *stdout_buff = oscap_acquire_pipe_to_string(pipefd[0]);
 			int wstatus;
 			waitpid(fork_result, &wstatus, 0);
 			_rule_add_info_message(rr, "Fix execution completed and returned: %d", WEXITSTATUS(wstatus));
 			if (stdout_buff != NULL && stdout_buff[0] != '\0')
 				_rule_add_info_message(rr, stdout_buff);
-			oscap_free(stdout_buff);
+			free(stdout_buff);
 			/* We return zero to indicate success. Rather than returning the exit code. */
 			result = 0;
 		}
 	} else {
 		_rule_add_info_message(rr, "Failed to fork. %s", strerror(errno));
-		oscap_free(temp_file);
+		free(temp_file);
 	}
 
 cleanup:
@@ -453,12 +526,13 @@ int xccdf_policy_remediate(struct xccdf_policy *policy, struct xccdf_result *res
 static const struct xccdf_fix *_find_fix_for_template(struct xccdf_policy *policy, struct xccdf_rule *rule, const char *template)
 {
 	struct xccdf_fix *fix = NULL;
-	struct oscap_list *fixes = _filter_fixes_by_applicability(policy, rule);
-	const struct _interpret_map map[] = {
-		{template, "Cloud!"},
-		{NULL, NULL}
-	};
-	fixes = _filter_fixes_by_system(fixes, _search_interpret_map, map);
+	struct oscap_list *fixes = _get_fixes(policy, rule);
+
+	if (template) {
+		const struct _interpret_map map[] = {	{template, "Cloud!"},
+							{NULL, NULL}};
+		fixes = _filter_fixes_by_system(fixes, _search_interpret_map, map);
+	}
 	fixes = _filter_fixes_by_distruption_and_reboot(fixes);
 	struct xccdf_fix_iterator *fix_it = oscap_iterator_new(fixes);
 	if (xccdf_fix_iterator_has_more(fix_it))
@@ -468,7 +542,42 @@ static const struct xccdf_fix *_find_fix_for_template(struct xccdf_policy *polic
 	return fix;
 }
 
-static inline int _xccdf_policy_rule_generate_fix(struct xccdf_policy *policy, struct xccdf_rule *rule, const char *template, int output_fd)
+static int _write_fix_header_to_fd(const char *sys, int output_fd, struct xccdf_rule *rule, unsigned int current, unsigned int total)
+{
+	if (oscap_streq(sys, "") || oscap_streq(sys, "urn:xccdf:fix:script:sh") || oscap_streq(sys, "urn:xccdf:fix:commands")) {
+		char *fix_header = oscap_sprintf(
+				"###############################################################################\n"
+				"# BEGIN fix (%i / %i) for '%s'\n"
+				"###############################################################################\n"
+				"(>&2 echo \"Remediating rule %i/%i: '%s'\")\n",
+				current, total, xccdf_rule_get_id(rule), current, total, xccdf_rule_get_id(rule));
+		return _write_text_to_fd_and_free(output_fd, fix_header);
+	} else {
+		return 0;
+	}
+}
+
+static int _write_fix_footer_to_fd(const char *sys, int output_fd, struct xccdf_rule *rule)
+{
+	if (oscap_streq(sys, "") || oscap_streq(sys, "urn:xccdf:fix:script:sh") || oscap_streq(sys, "urn:xccdf:fix:commands")) {
+		char *fix_footer = oscap_sprintf("# END fix for '%s'\n\n", xccdf_rule_get_id(rule));
+		return _write_text_to_fd_and_free(output_fd, fix_footer);
+	} else {
+		return 0;
+	}
+}
+
+static int _write_fix_missing_warning_to_fd(const char *sys, int output_fd, struct xccdf_rule *rule)
+{
+	if (oscap_streq(sys, "") || oscap_streq(sys, "urn:xccdf:fix:script:sh") || oscap_streq(sys, "urn:xccdf:fix:commands")) {
+		char *fix_footer = oscap_sprintf("# FIX FOR THIS RULE IS MISSING\n");
+		return _write_text_to_fd_and_free(output_fd, fix_footer);
+	} else {
+		return 0;
+	}
+}
+
+static inline int _xccdf_policy_rule_generate_fix(struct xccdf_policy *policy, struct xccdf_rule *rule, const char *template, int output_fd, unsigned int current, unsigned int total)
 {
 	// Ensure that given Rule is selected and applicable (CPE).
 	const bool is_selected = xccdf_policy_is_item_selected(policy, xccdf_rule_get_id(rule));
@@ -476,16 +585,18 @@ static inline int _xccdf_policy_rule_generate_fix(struct xccdf_policy *policy, s
 		dI("Skipping unselected Rule/@id=\"%s\"", xccdf_rule_get_id(rule));
 		return 0;
 	}
-	const bool is_applicable = xccdf_policy_model_item_is_applicable(xccdf_policy_get_model(policy), (struct xccdf_item*)rule);
-	if (!is_applicable) {
-		dI("Skipping notapplicable Rule/@id\"%s\"", xccdf_rule_get_id(rule));
-		return 0;
-	}
 	// Find the most suitable fix.
 	const struct xccdf_fix *fix = _find_fix_for_template(policy, rule, template);
 	if (fix == NULL) {
+		int ret = _write_fix_header_to_fd(template, output_fd, rule, current, total);
+		if (ret != 0)
+			return ret;
+		ret = _write_fix_missing_warning_to_fd(template, output_fd, rule);
+		if (ret != 0)
+			return ret;
+		ret = _write_fix_footer_to_fd(template, output_fd, rule);
 		dI("No fix element was found for Rule/@id=\"%s\"", xccdf_rule_get_id(rule));
-		return 0;
+		return ret;
 	}
 	dI("Processing a fix for Rule/@id=\"%s\"", xccdf_rule_get_id(rule));
 
@@ -508,30 +619,40 @@ static inline int _xccdf_policy_rule_generate_fix(struct xccdf_policy *policy, s
 	}
 	xccdf_fix_free(cfix);
 
-	// Print-out the fix to the output_fd
-	if (_write_text_to_fd_and_free(output_fd, fix_text) != 0) {
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, "write of the fix to fd=%d failed: %s", output_fd, strerror(errno));
-		return 1;
-	}
-	return 0;
+	int ret = _write_fix_header_to_fd(template, output_fd, rule, current, total);
+	if (ret != 0)
+		goto cleanup;
+	ret = _write_remediation_to_fd_and_free(output_fd, template, fix_text);
+	if (ret != 0)
+		goto cleanup;
+	fix_text = NULL;
+	ret = _write_fix_footer_to_fd(template, output_fd, rule);
+
+cleanup:
+	free(fix_text);
+	return ret;
 }
 
-static int _xccdf_policy_item_generate_fix(struct xccdf_policy *policy, struct xccdf_item *item, const char *template, int output_fd)
+static int _xccdf_item_recursive_gather_selected_rules(struct xccdf_policy *policy, struct xccdf_item *item, struct oscap_list *rule_list)
 {
 	int ret = 0;
+	const bool is_selected = xccdf_policy_is_item_selected(policy, xccdf_item_get_id(item));
+	if (!is_selected) {
+		return ret;
+	}
 	switch (xccdf_item_get_type(item)) {
 	case XCCDF_GROUP:{
 		struct xccdf_item_iterator *child_it = xccdf_group_get_content((struct xccdf_group *) item);
 		while (xccdf_item_iterator_has_more(child_it)) {
 			struct xccdf_item *child = xccdf_item_iterator_next(child_it);
-			ret = _xccdf_policy_item_generate_fix(policy, child, template, output_fd);
+			ret = _xccdf_item_recursive_gather_selected_rules(policy, child, rule_list);
 			if (ret != 0)
 				break;
 		}
 		xccdf_item_iterator_free(child_it);
 		} break;
 	case XCCDF_RULE:{
-		ret = _xccdf_policy_rule_generate_fix(policy, (struct xccdf_rule *) item, template, output_fd);
+		oscap_list_add(rule_list, (struct xccdf_rule *) item);
 		} break;
 	default:
 		assert(false);
@@ -540,31 +661,154 @@ static int _xccdf_policy_item_generate_fix(struct xccdf_policy *policy, struct x
 	return ret;
 }
 
+static int _write_script_header_to_fd(struct xccdf_policy *policy, struct xccdf_result *result, const char *sys, int output_fd)
+{
+
+	if (!(oscap_streq(sys, "") || oscap_streq(sys, "urn:xccdf:fix:script:sh") || oscap_streq(sys, "urn:xccdf:fix:commands") ||
+		oscap_streq(sys, "urn:xccdf:fix:script:ansible")))
+		return 0; // no header required
+
+	bool script = oscap_streq(sys, "urn:xccdf:fix:script:ansible");
+	const char *oscap_version = oscap_get_version();
+	char *fix_header;
+	const char *format = sys != NULL ? sys : "";
+	const char *template = sys != NULL ? " --template " : "";
+
+	if (result == NULL) {
+		// Profile-based remediation fix
+		struct xccdf_profile *profile = xccdf_policy_get_profile(policy);
+		const char *profile_id = xccdf_profile_get_id(profile);
+		// Title
+		struct oscap_text_iterator *title_iterator = xccdf_profile_get_title(profile);
+		char *profile_title = oscap_textlist_get_preferred_plaintext(title_iterator, NULL);
+		oscap_text_iterator_free(title_iterator);
+		// Description
+		struct oscap_text_iterator *description_iterator = xccdf_profile_get_description(profile);
+		char *profile_description = description_iterator != NULL ?
+				oscap_textlist_get_preferred_plaintext(description_iterator, NULL) : "Not available";
+		oscap_text_iterator_free(description_iterator);
+
+		struct xccdf_benchmark *benchmark = xccdf_policy_get_benchmark(policy);
+		const char *benchmark_version_info = xccdf_benchmark_get_version(benchmark);
+		const char *benchmark_id = xccdf_benchmark_get_id(benchmark);
+		const struct xccdf_version_info *xccdf_version = xccdf_benchmark_get_schema_version(benchmark);
+		const char *xccdf_version_name = xccdf_version_info_get_version(xccdf_version);
+
+		fix_header = oscap_sprintf(
+			"###############################################################################\n#\n"
+			"# %s remediation role for profile %s\n"
+			"# Profile Title:  %s\n"
+			"# Profile Description:  %s\n"
+			"#\n"
+			"# Benchmark ID:  %s\n"
+			"# Benchmark Version:  %s\n#\n"
+			"# XCCDF Version:  %s\n#\n"
+			"# This file was generated by OpenSCAP %s using:\n"
+			"# 	$ oscap xccdf generate fix --profile %s%s%s sds.xml \n#\n"
+			"# This script is generated from an OpenSCAP profile without preliminary evaluation.\n"
+			"# It attempts to fix every selected rule, even if the system is already compliant.\n#\n"
+			"###############################################################################\n\n",
+				script ? "Ansible" : "Bash", profile_id, profile_title, profile_description, benchmark_id,
+				benchmark_version_info, xccdf_version_name, oscap_version, profile_id, template, format);
+
+		free(profile_title);
+		free(profile_description);
+
+	} else {
+		// Results-based remediation fix
+		const char *start_time = xccdf_result_get_start_time(result);
+		const char *end_time = xccdf_result_get_end_time(result);
+		const char *result_id = xccdf_result_get_id(result);
+		const struct xccdf_version_info *xccdf_version = xccdf_result_get_schema_version(result);
+		const char *xccdf_version_name = xccdf_version_info_get_version(xccdf_version);
+
+		fix_header = oscap_sprintf(
+			"###############################################################################\n#\n"
+			"# %s remediation role for the results of evaluation of profile %s \n"
+			"# XCCDF Version:  %s\n#\n"
+			"# Evaluation Start Time:  %s\n"
+			"# Evaluation End Time:  %s\n#\n"
+			"# This file was generated by OpenSCAP %s using:\n"
+			"# 	$ oscap xccdf generate fix --result-id %s%s%s xccdf-results.xml \n#\n"
+			"# This script is generated from the results of a profile evaluation.\n"
+			"# It attempts to remediate all issues from the selected rules that failed the test.\n"
+			"#\n"
+			"###############################################################################\n\n",
+				script ? "Ansible" : "Bash", xccdf_result_get_profile(result), xccdf_version_name,
+				start_time != NULL ? start_time : "Unknown",
+				end_time, oscap_version, result_id, template, format);
+	}
+
+	if (script) {
+		char *ansible_fix_header = oscap_sprintf(
+			"---\n"
+			"%s\n"
+			"# - hosts: localhost # set required host\n"
+			"   tasks:\n",
+				fix_header);
+		free(fix_header);
+		return _write_text_to_fd_and_free(output_fd, ansible_fix_header);
+	} else {
+		return _write_text_to_fd_and_free(output_fd, fix_header);
+	}
+}
+
 int xccdf_policy_generate_fix(struct xccdf_policy *policy, struct xccdf_result *result, const char *sys, int output_fd)
 {
 	__attribute__nonnull__(policy);
+	int ret = 0;
 
+	struct oscap_list *rules_to_fix = oscap_list_new();
 	if (result == NULL) {
 		// No TestResult is available. Generate fix from the stock profile.
-		dI("Generating fixes for policy(profile/@id=%s)", xccdf_policy_get_id(policy));
-		int ret = 0;
+		dI("Generating profile-oriented fixes for policy(profile/@id=%s)", xccdf_policy_get_id(policy));
 		struct xccdf_benchmark *benchmark = xccdf_policy_get_benchmark(policy);
+
 		if (benchmark == NULL) {
 			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Could not find benchmark model for policy id='%s' when generating fixes.", xccdf_policy_get_id(policy));
 			return 1;
 		}
+
+		if (_write_script_header_to_fd(policy, result, sys, output_fd) != 0)
+			return 1;
+
 		struct xccdf_item_iterator *item_it = xccdf_benchmark_get_content(benchmark);
 		while (xccdf_item_iterator_has_more(item_it)) {
 			struct xccdf_item *item = xccdf_item_iterator_next(item_it);
-			ret = _xccdf_policy_item_generate_fix(policy, item, sys, output_fd);
+			ret = _xccdf_item_recursive_gather_selected_rules(policy, item, rules_to_fix);
 			if (ret != 0)
 				break;
 		}
 		xccdf_item_iterator_free(item_it);
-		return ret;
 	}
 	else {
-		// TODO.
-		return 1;
+		dI("Generating result-oriented fixes for policy(result/@id=%s)", xccdf_result_get_id(result));
+
+		if (_write_script_header_to_fd(policy, result, sys, output_fd) != 0)
+			return 1;
+
+		struct xccdf_rule_result_iterator *rr_it = xccdf_result_get_rule_results(result);
+		while (xccdf_rule_result_iterator_has_more(rr_it)) {
+			struct xccdf_rule_result *rr = xccdf_rule_result_iterator_next(rr_it);
+			if (xccdf_rule_result_get_result(rr) != XCCDF_RESULT_FAIL)
+				continue;
+			struct xccdf_rule *rule = _lookup_rule_by_rule_result(policy, rr);
+			oscap_list_add(rules_to_fix, rule);
+		}
+		xccdf_rule_result_iterator_free(rr_it);
 	}
+
+	const unsigned int total = oscap_list_get_itemcount(rules_to_fix);
+	unsigned int current = 1;
+	struct oscap_iterator *rules_to_fix_it = oscap_iterator_new(rules_to_fix);
+	while (oscap_iterator_has_more(rules_to_fix_it)) {
+		struct xccdf_rule *rule = (struct xccdf_rule*)oscap_iterator_next(rules_to_fix_it);
+		ret = _xccdf_policy_rule_generate_fix(policy, rule, sys, output_fd, current++, total);
+		if (ret != 0)
+			break;
+	}
+
+	oscap_iterator_free(rules_to_fix_it);
+	oscap_list_free(rules_to_fix, NULL);
+	return ret;
 }
