@@ -29,6 +29,7 @@
 #include "public/oscap.h"
 #include "alloc.h"
 #include <stdarg.h>
+#include <string.h>
 
 #ifndef __attribute__nonnull__
 #define __attribute__nonnull__(x) assert((x) != NULL)
@@ -172,7 +173,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 #define OSCAP_IGETTER_GEN(ITYPE,SNAME,MNAME) OSCAP_IGETTER(ITYPE,SNAME,MNAME) OSCAP_ITERATOR_GEN(ITYPE)
 
 /**
- * Generete a geter function from a hash table.
+ * Generate a getter function from a hash table.
  * Signature of the generated function will be as follows (substitute uppercase strings with actual params):
  * @code RTYPE SNAME_MNAME(const struct SNAME* item, const char* key); @endcode
  * @param RTYPE type of the hash table item
@@ -185,7 +186,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 	{ return oscap_htable_get(item->MEXP, key); }
 
 /**
- * Generete a geter function from a hash table.
+ * Generate a getter function from a hash table.
  * Signature of the generated function will be as follows (substitute uppercase strings with actual params):
  * @code RTYPE SNAME_MNAME(const struct SNAME* item, const char* key); @endcode
  * @param RTYPE type of the hash table item
@@ -195,7 +196,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 #define OSCAP_HGETTER(RTYPE,SNAME,MNAME) OSCAP_HGETTER_EXP(RTYPE,SNAME,MNAME,MNAME)
 
 /**
- * Generete a geter function from a hash table.
+ * Generate a getter function from a hash table.
  * Signature of the generated function will be as follows (substitute uppercase strings with actual params):
  * @code struct RTYPE* SNAME_MNAME(const struct SNAME* item, const char* key); @endcode
  * @param RTYPE type of the hash table item
@@ -207,7 +208,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 #define OSCAP_SETTER_HEADER(SNAME, MTYPE, MNAME) bool SNAME##_set_##MNAME(struct SNAME *obj, MTYPE newval)
 
 /**
- * Generete a setter function with a check.
+ * Generate a setter function with a check.
  * Signature of the generated function will be as follows (substitute uppercase strings with actual params):
  * @code bool SNAME_get_MNAME(struct SNAME *obj, MTYPE *item); @endcode
  * @param SNAME Name of the structure.
@@ -222,7 +223,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 	{ if (!(CHECK)) return false; DELETER(obj->MNAME); obj->MNAME = ASSIGNER(newval); return true; }
 
 /**
- * Generete a setter function without a check.
+ * Generate a setter function without a check.
  * @see OSCAP_SETTER_GENERIC_CHECK
  */
 #define OSCAP_SETTER_GENERIC(SNAME, MTYPE, MNAME, DELETER, ASSIGNER) \
@@ -230,7 +231,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 	{ DELETER(obj->MNAME); obj->MNAME = ASSIGNER(newval); return true; }
 
 /**
- * Generete a setter function without a check that does not delete the previous value.
+ * Generate a setter function without a check that does not delete the previous value.
  * @see OSCAP_SETTER_GENERIC_CHECK
  */
 #define OSCAP_SETTER_GENERIC_NODELETE(SNAME, MTYPE, MNAME, ASSIGNER) \
@@ -238,7 +239,7 @@ typedef void (*oscap_consumer_func) (void *, void *);
 	{ obj->MNAME = ASSIGNER(newval); return true; }
 
 /**
- * Generete a setter function using a simple assignment.
+ * Generate a setter function using a simple assignment.
  * @see OSCAP_SETTER_GENERIC_CHECK
  */
 #define OSCAP_SETTER_SIMPLE(SNAME, MTYPE, MNAME) \
@@ -307,8 +308,8 @@ typedef void (*oscap_consumer_func) (void *, void *);
  * the default value for strings not defined elsewhere.
  */
 struct oscap_string_map {
-	int value;		/* integer/enum value */
-	const char *string;	/* string representation of the value */
+	const int value;    /* integer/enum value */
+	const char *string; /* string representation of the value */
 };
 
 /**
@@ -331,15 +332,16 @@ const char *oscap_enum_to_string(const struct oscap_string_map *map, int val);
  * Use strdup on string, if string is NULL, return NULL
  * @param str String we want to duplicate
  */
-char *oscap_strdup(const char *str);
+static inline char *oscap_strdup(const char *str) {
+	if (str == NULL)
+		return NULL;
 
-/**
- * Use strtol on string, if string is NULL, return NaN
- * @param str String we want to duplicate
- * @param endptr see man strtol
- * @param base see man strtol
- */
-float oscap_strtol(const char *str, char **endptr, int base);
+#ifdef _MSC_VER
+	return _strdup(str);
+#else
+	return strdup(str);
+#endif
+}
 
 /**
  * Split a string.
@@ -352,12 +354,31 @@ float oscap_strtol(const char *str, char **endptr, int base);
 char **oscap_split(char *str, const char *delim);
 
 
-/// Use strcmp on strings, NULL safe
-int oscap_strcmp(const char *s1, const char *s2);
-/// Check for string equality
-bool oscap_streq(const char *s1, const char *s2);
-bool oscap_str_startswith(const char *str, const char *with);
-bool oscap_str_endswith(const char *str, const char *with);
+/// Just like strcmp except it's NULL-safe. Use the standard strcmp directly if possible.
+static inline int oscap_strcmp(const char *s1, const char *s2) {
+	if (s1 == NULL) s1 = "";
+	if (s2 == NULL) s2 = "";
+	return strcmp(s1, s2);
+}
+
+/// Check for string equality. Use the standard strcmp directly if possible.
+static inline bool oscap_streq(const char *s1, const char *s2) {
+	return oscap_strcmp(s1, s2) == 0;
+}
+
+/// Check whether str starts with "prefix"
+static inline bool oscap_str_startswith(const char *str, const char *prefix) {
+	return strncmp(str, prefix, strlen(prefix)) == 0;
+}
+
+/// Check whether str ends with "suffix"
+static inline bool oscap_str_endswith(const char *str, const char *suffix) {
+	const size_t str_len = strlen(str);
+	const size_t suffix_len = strlen(suffix);
+	if (suffix_len > str_len)
+		return false;
+	return strncmp(str + str_len - suffix_len, suffix, suffix_len) == 0;
+}
 /// Trim whitespace (modifies its argument!)
 char *oscap_trim(char *str);
 /// Print to a newly allocated string using a va_list.
@@ -381,15 +402,6 @@ void oscap_strtoupper(char *str);
 // check pointer equality
 bool oscap_ptr_cmp(void *node1, void *node2);
 
-/**
- * find file with given name and mode in given paths
- * @param filename filename to be found
- * @param mode desired file mode (e.g. R_OK for readable files)
- * @param pathvar environment variable to check for path, can be NULL
- * @param path path where to search the file
- */
-char *oscap_find_file(const char *filename, int mode, const char *pathvar, const char *path);
-
 /**
  * A helper function to expand given shorthand IPv6
  *
