@@ -61,6 +61,9 @@ function test_rhel {
             elif rpm -q --queryformat "%{VERSION}" --whatprovides redhat-release | grep ${RHEL_VERSION}'\.9'; then
                 # Workaround alpha and beta releases of Red Hat Enterprise Linux
                 EXPECTED_NA=0
+            elif rpm -q --queryformat "%{VENDOR}" --whatprovides redhat-release | grep "Oracle"; then
+                # Workaround OL which also has redhat-release package installed
+                EXPECTED_NA=1
             elif echo "$RHEL_RELEASE" | grep "\.el${RHEL_VERSION}[._]"; then
                 EXPECTED_NA=0
             elif echo "$RHEL_RELEASE" | grep "\.ael${RHEL_VERSION}b"; then
@@ -103,6 +106,7 @@ test_run "test_api_xccdf_default_cpe_fedora19" test_fedora 19
 test_run "test_api_xccdf_default_cpe_fedora20" test_fedora 20
 test_run "test_api_xccdf_default_cpe_fedora21" test_fedora 21
 test_run "test_api_xccdf_default_cpe_fedora23" test_fedora 23
+test_run "test_api_xccdf_default_cpe_fedora24" test_fedora 24
 test_run "test_api_xccdf_default_cpe_rhel5" test_rhel 5
 test_run "test_api_xccdf_default_cpe_rhel6" test_rhel 6
 test_run "test_api_xccdf_default_cpe_rhel7" test_rhel 7
