@@ -12,7 +12,7 @@
 #      David Niemoller
 #      Ondrej Moris, <omoris@redhat.com>
 
-. ${srcdir}/../../test_common.sh
+. ../../test_common.sh
 
 # Test cases.
 
