@@ -12,9 +12,7 @@
 #      David Niemoller
 #      Ondrej Moris, <omoris@redhat.com>
 
-. ${srcdir}/../../test_common.sh
-
-# Test Cases.
+. ../../test_common.sh
 
 function test_probes_textfilecontent54 {
 
@@ -36,7 +34,7 @@ function test_probes_textfilecontent54 {
     echo "valid_key = valid_value" > "$FILE_A"
     echo "valid_key = valid_value" >> "$FILE_A"
 
-    ../../../utils/.libs/oscap oval eval --results $RF $DF
+    $OSCAP oval eval --results $RF $DF
     
     if [ -f $RF ]; then
 	verify_results "def" $DF $RF 13 && verify_results "tst" $DF $RF 16
@@ -45,13 +43,9 @@ function test_probes_textfilecontent54 {
 	ret_val=1
     fi
 
+    rm -f $FILE_A $FILE_B $FILE_C
+
     return $ret_val
 }
 
-# Testing.
-
-test_init "test_probes_textfilecontent54.log"
-
-test_run "test_probes_textfilencontent54" test_probes_textfilecontent54
-
-test_exit
+test_probes_textfilecontent54
